<?php
/**
 * webEdition CMS
 *
 * $Rev: 15571 $
 * $Author: mokraemer $
 * $Date: 2018-08-09 21:52:14 +0200 (Thu, 09 Aug 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 *
 * webEdition configuration script
 *
 * optional URL parameters:
 * - debug		turn on debug messages
 * - debugoff	turn off debug messages
 * - phpinfo	show phpinfo() instead of the setup screen
 * example:		http://yourdomain/setup.php?debug
 * 				http://yourdomain/setup.php?phpinfo
 */
// some utility features:
ini_set('display_errors', 'Off');
ini_set('error_reporting', E_ALL);

if(isset($_REQUEST['phpinfo'])){
	phpinfo();
	exit();
}
if(version_compare(PHP_VERSION, '5.6.0', '<')){
	die('<html><head><title>webEdition setup</title></head><body><div style="font-family:sans-serif, sans; font-size:10pt; border:1px solid red; text-align:center; padding:10px; margin:100px;"><b>PHP Version mismatch</b><br /><br />The PHP Version currently used  on this server is too old to run webEdition.<br />webEdition versions newer than need at least PHP Version 5.6.0, please ask your administrator to update your PHP installation!</div></body></html>');
}

// first some includes:
if(
	!is_readable('./webEdition/we/include/we_version.php') ||
	!is_readable('./webEdition/we/include/conf/we_conf.inc.php.default') ||
	!is_readable('./webEdition/we/include/conf/we_conf_global.inc.php.default') ||
	!is_dir('./webEdition')
){
	die("No webEdition installation found. This script has to be placed in your DOCUMENT_ROOT besides your webEdition folder!");
}
if(!is_readable('./webEdition/we/include/conf/we_conf.inc.php')){
	copy('./webEdition/we/include/conf/we_conf.inc.php.default', './webEdition/we/include/conf/we_conf.inc.php');
}
if(!is_readable('./webEdition/we/include/conf/we_conf_global.inc.php')){
	copy('./webEdition/we/include/conf/we_conf_global.inc.php.default', './webEdition/we/include/conf/we_conf_global.inc.php');
}
if(!defined('DB_CONNECT')){
	define('DB_CONNECT', 'mysqli_connect');
}

include_once './webEdition/we/include/we_version.php';
include_once './webEdition/lib/we/core/autoload.php';
@session_start();
if(isset($_REQUEST["debug"]) && !isset($_SESSION["debug"])){
	$_SESSION["debug"] = true;
}
if(isset($_REQUEST["debugoff"]) && isset($_SESSION["debug"])){
	unset($_SESSION["debug"]);
}

// boolean for error state (for disabling the next button if any errors occured)
$errors = false;
$steps = [["id" => "1",
	"title" => "webEdition setup",
	"name" => "welcome"
	],
	["id" => "2",
		"title" => "System requirements",
		"name" => "requirements"
	],
	["id" => "3",
		"title" => "Filesystem checks",
		"name" => "filesystem"
	],
	["id" => "4",
		"title" => "Database settings",
		"name" => "database"
	],
	["id" => "5",
		"title" => "Check database settings",
		"name" => "databasecheck"
	],
	["id" => "6",
		"title" => "Language selection",
		"name" => "language"
	],
	["id" => "7",
		"title" => "Summary",
		"name" => "summary"
	],
	["id" => "8",
		"title" => "Installation",
		"name" => "installation"
	],
	["id" => "9",
		"title" => "Setup complete.",
		"name" => "finish"
	],
	["id" => "10",
		"title" => "Cleanup complete.",
		"name" => "cleanup"
	],
];

// identify current step:
$currentStep = (isset($_REQUEST["step"]) && !empty($_REQUEST["step"]) && intval($_REQUEST["step"]) >= 1 && intval($_REQUEST["step"]) <= count($steps) ?
	$steps[intval($_REQUEST["step"]) - 1] : $steps[0]);

//functions for checking system
function ini_get_bool($val){
	$bool = ini_get($val);
	if($val == "1"){
		return true;
	}
	if($val == "0"){
		return false;
	}
	switch(strtolower($bool)){
		case '1':
		case 'on':
		case 'yes':
		case 'true':
			return true;
		default:
			return false;
	}
	return false;
}

// function for executing steps:
function step_welcome(){
	$output = '<b>Welcome to webEdition!</b><br />
We recommend to use the latest OnlineInstaller instead of this <br/>tarball setup, because the OnlineInstaller has much more features<br/> and is much faster.<br/>

The OnlineInstaller is available at <a href="http://download.webedition.org/releases/" target="_blank">download.webedition.org/releases/</a>
or at <a href="https://sourceforge.net/projects/webedition/files/webEdition%20OnlineInstaller/" target="_blank">sourceforge.net/projects/webedition/files/webEdition%20OnlineInstaller/</a><br/><br/><br/>
This webEdition tarball setup script will guide you through the initial configuration steps:
<ul>
		<li>System requirements and recommendations</li>
		<li>Filesystem checks (write permissions etc.)</li>
		<li>Database configuration and checks</li>
		<li>Language and character set</li>
		<li>Database installation and webEdition configuration</li>
</ul>
<b>Important:</b> Please remember to delete this script afterwards in order to prevent system damage by misuse!
<br /><br />
	';
	// session and cookie test:
	$sessionid = session_id();
	if(!$sessionid){
		$_SESSION["we_test"] = @session_id();
		$_COOKIE["we_test"] = @session_id();
	} else {
		$_SESSION["we_test"] = "";
		$_COOKIE["we_test"] = "";
	}
	return $output;
}

function step_requirements(){
	global $errors;

	$sdkDbOK = true;
	$phpExtensionsDetectable = true;

	$phpextensions = get_loaded_extensions();
	foreach($phpextensions as &$extens){
		$extens = strtolower($extens);
	}
	$phpextensionsMissing = array();
	$phpextensionsMin = array('ctype', 'date', 'dom', 'filter', 'gd', 'iconv', 'libxml', 'mbstring', 'mysqli', 'pcre', 'Reflection', 'session', 'SimpleXML', 'SPL',
		'standard', 'tokenizer', 'xml', 'zlib');

	if(count($phpextensions) > 3){
		foreach($phpextensionsMin as $exten){
			if(!in_array(strtolower($exten), $phpextensions, true)){
				$phpextensionsMissing[] = $exten;
			}
		}
		/* if ( in_array(strtolower('PDO'),$phpextensions) && in_array(strtolower('pdo_mysql'),$phpextensions) ){//später ODER mysqli
		  $phpextensionsSDK_DB = 'PDO &amp; PDO_mysql';
		  } else {
		  $phpextensionsSDK_DB= '';
		  $sdkDbOK = false;
		  } */
	} else {
		$phpExtensionsDetectable = false;
	}

	$output = "Checking if all system requirements are met. Some additional tests are performed as they are needed for webEdition to be fully functional but are not essential to run webEdition.<br /><br /><b>Basic Requirements:</b><ul style=\"list-style-position:outside;\">";
	$errors = false;
	if(version_compare(PHP_VERSION, "5.6.0", "<")){
		$output .= tpl_error("PHP Version 5.6.0 or newer required for webEdition versions never than 6.3.9.0!");
		$errors = true;
	} else {
		$output .= tpl_ok("Your PHP Version is up to date (Version " . PHP_VERSION . ")");
	}
	if(!empty($phpextensionsMissing)){
		$output .= tpl_error("Required PHP extensions are not available, missing: " . implode(', ', $phpextensionsMissing));
		$errors = true;
	}
	if(!is_callable("mysqli_query")){
		$output .= tpl_error("PHP MySQL Support is required for running webEdition! MySQL servers at version 5.0 or newer are supported.");
		$errors = true;
	} else {
		$mysqlVersion = mysqli_get_client_version();
		if($mysqlVersion < 50000){
			$output .= tpl_error("MySQL Version 5.0 or newer required!");
			$errors = true;
		} else {
			$output .= tpl_ok("PHP MySQL support available (Client API Version " . $mysqlVersion . " found)");
		}
	}

	$output .= "</ul><b>Additional requirements:</b><ul style=\"list-style-position:outside;\">";
	if(ini_get_bool('safe_mode')){
		$output .= tpl_warning("PHP Safe Mode is active.<br />webEdition may run with activated <a href=\"http://www.php.net/manual/en/features.safe-mode.php\" target=\"_blank\">PHP Safe Mode</a>, yet we do not recommend it since it is DEPRECATED since PHP version 5.3. We also cannot guarantee that all features of webEdition will work properly.");
	}
	if(ini_get_bool('register_globals')){
		$output .= tpl_warning("register_globals is active!<br />This may cause <b>severe security problems</b>, is declared DEPRECATED since PHP version 5.3 and we strongly recommend to disable this \"feature\". See <a href=\"http://www.php.net/manual/en/security.globals.php\" target=\"_blank\">php.net/manual</a> for more information.");
	}
	if(in_array('suhosin', get_loaded_extensions())){
		$output .= tpl_warning("Suhosin is active! The application <b>might</b> work with activated <a href=\"http://www.hardened-php.net/\" target=\"_blank\">Suhosin</a>, but yet we do not recommend it, since Suhosin can lead to problems due it's many configuration options.");
	}


	if(!is_callable("curl_getinfo")){
		$output .= tpl_warning("curl support is not available.<br />You need at least curl or allow_url_fopen activated for using webEdition liveUpdate, the First Steps Wizard or the application installer.");
	} else {
		$curlVersion = curl_version();
		$output .= tpl_ok("curl support is available (Version " . $curlVersion["version"] . " found)");
	}
	if(!ini_get_bool("allow_url_fopen")){
		$output .= tpl_warning("allow_url_fopen deactivated.<br />You need at least curl or allow_url_fopen activated for using webEdition liveUpdate.");
	} else {
		$output .= tpl_ok("allow_url_fopen activated.");
	}

	if(!is_callable("mb_convert_encoding")){
		$output .= tpl_warning("PHP multibyte functions not available");
	} else {
		$output .= tpl_ok("PHP multibyte functions available");
	}
	if(!is_callable("gd_info")){
		$output .= tpl_warning("gdlib functions not available");
	} else {
		$output .= tpl_ok("gdlib functions available (Version " . GD_VERSION . " found)");
	}
	if(!is_callable("exif_imagetype")){
		$output .= tpl_warning("exif extension not available: EXIF-Metadata for images are not available");
	}
	if(!$sdkDbOK){
		$output .= tpl_warning("SDK Operations and WE-APPS with database access are not available");
	}
	if(!$phpExtensionsDetectable){
		$output .= tpl_warning("Not all requirements could be checked (Suhosin?). Please check the system requirements at http://documentation.webedition.org/wiki/de/webedition/system-requirements/start");
	}

	if(defined("PCRE_VERSION") && substr(PCRE_VERSION, 0, 1) < 7){
		$output .= tpl_warning('Your PCRE extension is outdated: ' . PCRE_VERSION . " detected. This can lead to problems, particularly in future webEdition versions.");
	}
	if(!defined("PCRE_VERSION")){
		$output .= tpl_warning("Your PCRE extension version can not be determined. Versions before 7.0 can lead to problems, particularly in future webEdition versions.");
	}



	$output .= "</ul>";
	if($errors === true){
		$output .= tpl_errorbox("Some of the essential system requirements are not met. Please check the informations given above and update yor system!<br /><a href=\"?phpinfo\" target=\"_blank\">Click here</a> to check your system's PHP configuration.");
	}
	// session and cookie test:
	$output .= "</ul><b>Session / cookie test:</b><ul style=\"list-style-position:outside;\">";
	if(isset($_SESSION["we_test"]) && $_SESSION["we_test"] == @session_id()){
		$output .= tpl_error("Session test failed. Maybe restarting your browser may help.");
	} else {
		$output .= tpl_ok("Session test");
	}
	if(isset($_COOKIE["we_test"]) && $_COOKIE["we_test"] == @session_id()){
		$output .= tpl_error("Cookie test failed. Maybe cookies are disabled in your browser.");
	} else {
		$output .= tpl_ok("Cookie test");
	}
	$output .= "</ul>";
	return $output;
}

function step_filesystem(){
	global $errors;
	$output = "Some Directories and files have to be writable by the webserver for running webEdition:<ul>";
	if(!is_writable('./')){
		$output .= tpl_error("DOCUMENT ROOT is not writable!");
		$errors = true;
	} else {
		$output .= tpl_ok("./ (DOCUMENT_ROOT)");
	}
	if(!is_writable('./webEdition/')){
		$output .= tpl_error("The directory webEdition/ is not writable!");
		$errors = true;
	} else {
		$output .= tpl_ok("./webEdition");
	}

	// check if directory exists
	if(!is_dir('./webEdition/site')){
		mkdir('./webEdition/site');
	}
	if(!is_writable('./webEdition/site')){
		$output .= tpl_error("The directory webEdition/site could not be created or is not writable!");
		$errors = true;
	} else {
		$output .= tpl_ok("./webEdition/site");
	}

	// check if directory exists
	if(!is_dir('./webEdition/we/templates')){
		mkdir('./webEdition/we/templates');
	}
	if(!is_writable('./webEdition/we/templates')){
		$output .= tpl_error("The directory webEdition/we/templates could not be created or is not writable!");
		$errors = true;
	} else {
		$output .= tpl_ok("./webEdition/we/templates");
	}

	if(!is_writable('./webEdition/we/include/conf')){
		$output .= tpl_error("The webEdition configuration directory webEdition/we/include/conf is not writable!");
		$errors = true;
	} else {
		$output .= tpl_ok("./webEdition/we/include/conf");
	}
	if(!is_writable('./webEdition/we/include/conf/we_conf.inc.php')){
		$output .= tpl_error("The webEdition configuration file webEdition/we/include/conf/we_conf.inc.php is not writable!");
		$errors = true;
	} else {
		$output .= tpl_ok("./webEdition/we/include/conf/we_conf.inc.php");
	}

	// check if directory exists
	if(!is_dir('./webEdition/we/tmp')){
		mkdir('./webEdition/we/tmp');
	}
	if(!is_writable('./webEdition/we/tmp')){
		$output .= tpl_error("The webEdition temporary directory webEdition/we/tmp could not be created or is not writable!");
		$errors = true;
	} else {
		$output .= tpl_ok("./webEdition/we/tmp");
	}
	if(!is_dir('./webEdition/liveUpdate/tmp')){
		mkdir('./webEdition/liveUpdate/tmp');
	}

	if(!is_writable('./webEdition/liveUpdate/tmp')){
		$output .= tpl_warning("The webEdition liveUpdate temporary directory webEdition/liveUpdate/tmp could not be created or is not writable! You will not be able to use this feature.");
	} else {
		$output .= tpl_ok("./webEdition/liveUpdate/tmp");
	}

	$output .= "</ul>";
	if($errors === true){
		$output .= tpl_errorbox("There were some errors regarding file access privileges. Please fix these issues (i.e. via ftp) and try again.<br/> We do not recommend to try to make these files and directories writeble on an individual base because there are additional files and directories which ahve to be writeble and which are not testet here.<br/> You should use your ftp programm to set the rights to 755 to the /webEdition and all included directories and files recursively (good ftp programms allow to set the rights in a singel step)");
	} else {
		$output .= "All these directories seem to be writable by the webserver.<br /><br />";
	}
	$output .= "Sometimes there may occur problems while using webEdition regarding file access permissions, even if the directories seem to be writable to this script. If that happens you should verify all access privileges and file owner informations of the critical webEdition directories. This can be done with ftp applications like <a href=\"http://www.filezilla-project.org\" target=\"_blank\">FileZilla</a>.";
	return $output;
}

function step_database(){
	// database host name
	$input_host = '<input style="width:200px:" name="db_host" value="' . (isset($_SESSION["db_host"]) && !empty($_SESSION["db_host"]) ? $_SESSION["db_host"] : 'localhost') . '"/>';

	// database name:
	$input_database = '<input style="width:200px:" name="db_database" value="' . (isset($_SESSION["db_database"]) && !empty($_SESSION["db_database"]) ? $_SESSION["db_database"] : 'webedition') . '"/>';

	// table prefix:
	$input_tableprefix = '<input style="width:200px:" name="db_tableprefix" value="' . (isset($_SESSION["db_tableprefix"]) && !empty($_SESSION["db_tableprefix"]) ? $_SESSION["db_tableprefix"] : '') . '"/>';

	// database username:
	$input_username = '<input style="width:200px:" name="db_username" value="' . (isset($_SESSION["db_username"]) && !empty($_SESSION["db_username"]) ? $_SESSION["db_username"] : '') . '"/>';

	// database user password:
	$input_password = '<input  class="small" type="password" style="width:200px:" name="db_password" value="' . (isset($_SESSION["db_password"]) ? $_SESSION["db_password"] : '') . '"/>';

	return '
		Please enter all informations required to connect to the database server:<br /><br />
		<table class="small">
		<tr><td style="width:80px;">Server: </td><td>' . $input_host . '</td></tr>
		<tr><td style="width:80px;">Database: </td><td>' . $input_database . '</td></tr>
		<tr><td style="width:80px;">Table prefix: </td><td>' . $input_tableprefix . '</td></tr>
		<tr><td style="width:80px;">Username: </td><td>' . $input_username . '</td></tr>
		<tr><td style="width:80px;">Password: </td><td>' . $input_password . '</td></tr>
	</table>';
}

function step_databasecheck(){
	global $errors;
	$output = "Some checks are being performed to verify that the database server is fully operational:<ul>";
	if((!isset($_SESSION["db_host"]) || empty($_SESSION["db_host"])) && (!isset($_REQUEST["db_host"]) || empty($_REQUEST["db_host"]))){
		$output .= tpl_error("Please enter the host name of your MySQL database server.");
		$errors = true;
	} else if(isset($_REQUEST["db_host"])){
		//$_SESSION["db_host"] = str_replace("/*","",str_replace('"','',str_replace("'","",trim($_REQUEST["db_host"]))));
		$_SESSION["db_host"] = $_REQUEST["db_host"];
	}
	if((!isset($_SESSION["db_database"]) || empty($_SESSION["db_database"])) && (!isset($_REQUEST["db_database"]) || empty($_REQUEST["db_database"]))){
		$output .= tpl_error("Please enter the database name to be used by webEdition. This database does not need to exist yet, if the specified database user has the permission to create databases.");
		$errors = true;
	} else if(isset($_REQUEST["db_database"])){
		//$_SESSION["db_database"] = str_replace("/*","",str_replace('"','',str_replace("'","",trim($_REQUEST["db_database"]))));
		$_SESSION["db_database"] = $_REQUEST["db_database"];
	}
	/*
	  if(isset($_REQUEST["db_tableprefix"])) {
	  $_SESSION["db_tableprefix"] = str_replace("/*","",str_replace('"','',str_replace("'","",trim($_REQUEST["db_tableprefix"]))));
	  } else if(!isset($_SESSION["db_tableprefix"])) {
	  $_SESSION["db_tableprefix"] = '';
	  }
	 */
	if(isset($_REQUEST["db_tableprefix"]) && preg_match('/^[a-z0-9_-]{0,}$/i', $_REQUEST["db_tableprefix"])){
		$_SESSION["db_tableprefix"] = $_REQUEST["db_tableprefix"];
	} else if(!isset($_SESSION["db_tableprefix"])){
		$_SESSION["db_tableprefix"] = '';
	}

	if((!isset($_SESSION["db_username"]) || empty($_SESSION["db_username"])) && (!isset($_REQUEST["db_username"]) || empty($_REQUEST["db_username"]))){
		$output .= tpl_error("Please enter the username for accessing your MySQL database server.");
		$errors = true;
	} else if(isset($_REQUEST["db_username"])){
		//$_SESSION["db_username"] = str_replace("/*","",str_replace('"','',str_replace("'","",trim($_REQUEST["db_username"]))));
		$_SESSION["db_username"] = $_REQUEST["db_username"];
	}
	if(!isset($_SESSION["db_password"]) && !isset($_REQUEST["db_password"])){
		$output .= tpl_error("Please enter the password for accessing your MySQL database server.");
		$errors = true;
	} else if(isset($_REQUEST["db_password"])){
		//$_SESSION["db_password"] = str_replace("/*","",str_replace('"','',str_replace("'","",trim($_REQUEST["db_password"]))));
		$_SESSION["db_password"] = $_REQUEST["db_password"];
		if(empty($_REQUEST["db_password"])){
			$output .= tpl_warning("No password entered. Are you sure?");
		}
	}
	if($errors){
		$output .= tpl_errorbox("Please enter the missing informations.");
		return $output . '</ul>';
	}

	// check connection to db server using the entered data
	$conn = mysqli_connect($_SESSION["db_host"], $_SESSION["db_username"], $_SESSION["db_password"], $_SESSION['db_database']);
	if(!$conn){
		$output .= tpl_error("Could not connect to MySQL database server: " . mysqli_connect_error());
		$errors = true;
		return $output . '</ul>';
	} else {
		$output .= tpl_ok("Connection test succeeded");
	}

	// check if selected database already exists:
	$op_createdb = false;
	//$result = @mysql_list_dbs($conn);
//	$result = mysqli_query($conn, "USE " . $_SESSION['db_database']);
	//$dblist = mysql_fetch_array($result);
	//$output .= print_r($dblist,true);
	//if(!in_array($_SESSION["db_database"],$dblist)) {
	/* if(!$result){
	  $output .= tpl_info("The database \"" . $_SESSION["db_database"] . "\" does not exist yet. Will try to create it.");
	  $op_createdb = true;
	  } else {
	  $output .= tpl_ok("The database \"" . $_SESSION["db_database"] . "\" exists already");
	  $op_createdb = false;
	  }

	  // try to create db:
	  if($op_createdb === true){
	  if(!mysqli_query($conn, "USE " . $_SESSION['db_database'])){
	  $output .= tpl_error("Could not create the database. Message from the server: " . mysqli_error($conn));
	  $errors = true;
	  return $output . '</ul>';
	  }
	  }
	  $result = mysqli_query($conn, "USE " . $_SESSION['db_database']);
	 */
	// check if there is already a webEdition installation present:

	$result = mysqli_query($conn, 'SELECT ID FROM ' . $_SESSION["db_tableprefix"] . 'tblUser');
	if(!$result){
		$output .= tpl_ok("The selected database obviously does not conain any previous webEdition installations using this table prefix");
	} else {
		$data = mysqli_num_rows($result);
		if(!empty($data)){
			$output .= tpl_warning("There is obviously a previous webEdition installation in the selected database. <b>All data will be lost if you continue this installation!</b> Please backup your data or use an alternate table prefix.");
		} else {
			$output .= tpl_ok("The selected database obviously does not conain any previous webEdition installations using this table prefix");
		}
	}
	if((float) mysqli_get_server_info($conn) < 5.5){
		$output .= tpl_warning(sprintf("The database server reports the version %s, webEdition requires at least the  MySQL-Server version 5.5. webEdition may work with the used version, but this can not be guarented for new webEdition versions (i.e. after updates). For webEdition version 7,  MySQL version 5 will definitely be required.<br/><span style=\"color:red;font-weight:bold\">In addition: In addition: The installed MySQL version is outdated. There are no security updates available for this version, which may put the security of the whole system at risk!</span><br/<br/>", mysqli_get_server_info($conn)));
	}

	// check for required database access permissions (select, insert, alter, update, drop)
	$output .= '</ul>Performing some permission tests for important database operations:<ul>';
	if(!mysqli_query($conn, "CREATE TABLE  `we_installer_test` (`id` VARCHAR( 100 ) NOT NULL) ENGINE = MyISAM;")){
		$output .= tpl_error("CREATE TABLE failed: " . mysqli_error($conn));
		$errors = true;
	} else {
		$output .= tpl_ok("CREATE TABLE succeeded");
	}
	if(!mysqli_query($conn, "INSERT INTO we_installer_test VALUES('eins');")){
		$output .= tpl_error("INSERT failed: " . mysqli_error($conn));
		$errors = true;
	} else {
		$output .= tpl_ok('INSERT succeeded');
	}
	if(!mysqli_query($conn, "UPDATE we_installer_test SET id='zwei' WHERE id!='zwei'")){
		$output .= tpl_error("UPDATE failed: " . mysqli_error($conn));
		$errors = true;
	} else {
		$output .= tpl_ok("UPDATE succeeded");
	}
	if(!mysqli_query($conn, "DROP TABLE `we_installer_test`;")){
		$output .= tpl_error("DROP TABLE failed: " . mysqli_error($conn));
		$errors = true;
	} else {
		$output .= tpl_ok("DROP TABLE succeeded");
	}



	$output .= "</ul>" .
		($errors === false ?
		"All seems to be ok, all requirements are met." :
		tpl_errorbox("There were some problems with the MySQL database server, please check the informations given above and fix these issues to continue the webEdition installation.")
		);


	return $output;
}

function step_language(){
	global $errors;
	$output = "Please select a language to be used by webEdition. You can change this at any time using the webEdition preferences dialog window. There you can alos change to an ISO backend charset";
	if(!is_dir('./webEdition/we/include/we_language/')){
		$output .= tpl_errorbox('There is a problem with your webEdition installation, could not find the language directory. Please verify that the installation archive has been completely unpacked into this directory.');
		$errors = true;
		return $output;
	}
	$langdirs = scandir('./webEdition/we/include/we_language/');
	//$output .= print_r($langdirs,true);
	$_language = array();
	foreach($langdirs as $lang){
		if(substr($lang, 0, 1) != "."){
			$_language["translation"][$lang] = $lang;
		}
	}
	asort($_language["translation"]);
	$defaultLanguage = "English";
	$defaultLanguageTranslation = "English";
	if(!isset($_SESSION["we_language_translation"])){
		$currentLanguage = $defaultLanguageTranslation;
	} else {
		$currentLanguage = $_SESSION["we_language_translation"];
	}
	$output .= '<input type="hidden" name="we_language_translation" value="' . $currentLanguage . '" />
<div style="display:block; margin:10px; text-align:center;"><select name="we_language" onchange="document.getElementsByName(\'we_language_translation\')[0].value = this[this.selectedIndex].text;">';
	foreach($_language["translation"] as $k => $v){
		if(!isset($_SESSION["we_language"]) && $k == $defaultLanguage){
			$selected = 'selected="selected" ';
		} else if(isset($_SESSION["we_language"]) && $_SESSION["we_language"] == $k){
			$selected = 'selected="selected" ';
		} else {
			$selected = "";
		}
		$output .= '<option ' . $selected . 'name="' . $v . '" value="' . $k . '">' . $v . '</option>';
	}
	$output .= '</select></div>' .
		// additional information box for iso encoded languages:

		"If your language is missing in this list, feel free to contribute a new translation to the webEdition community. You can find more informations about contributing code and translations on the <a href=\"http://www.webedition.org\" target=\"_blank\">webEdition website</a>.";

	$conn = mysqli_connect($_SESSION["db_host"], $_SESSION["db_username"], $_SESSION["db_password"]);
	mysqli_query($conn, "USE " . $_SESSION["db_database"]);
	mysqli_query($conn, "SHOW COLLATION WHERE Compiled = 'Yes' ");
	if(!isset($_SESSION["we_db_collation"])){
		$currentcharset = 'utf8_unicode_ci';
	} else {
		$currentcharset = $_SESSION["we_db_collation"];
	}

	$output .= "<br/>&nbsp;<br/>Please select the default encoding and the corresponding database collation (defining the standard sorting within database calls). We recommend utf8_unicode_ci" .
		'<input type="hidden" name="we_db_collation" value="' . $currentcharset . '" />' .
		'<div style="display:block; margin:10px; text-align:center;">' .
		'<select name="we_db_char" onchange="document.getElementsByName(\'we_db_collation\')[0].value = this[this.selectedIndex].text;">';
	$cset = '';
	while($row = mysqli_fetch_assoc($result)){
		if($cset != $row['Charset']){
			if($cset != ''){
				$output .= '</optgroup>';
			}
			$output .= '<optgroup label="' . $row['Charset'] . '">';
			$cset = $row['Charset'];
		}
		$output .= '<option ';
		if($row['Collation'] == $currentcharset){
			$output .= 'selected="selected"';
		}
		$output .= ' >' . $row['Collation'] . '</option>';
	}
	//sort($charsets);
	$output .= '</select></div>';

	return $output;
}

function step_summary(){
	global $errors;
	$output = "";
	if((!isset($_SESSION["we_language"]) || empty($_SESSION["we_language"])) && (!isset($_REQUEST["we_language"]) || empty($_REQUEST["we_language"]))){
		$output .= tpl_errorbox("Please select a valid language to be used by webEdition.");
		$errors = true;
	} else if(isset($_REQUEST["we_language"])){
		$_SESSION["we_language"] = str_replace(["/*", '"', "'"], "", trim($_REQUEST["we_language"]));
		$_SESSION["we_language_translation"] = str_replace(["/*", '"', "'"], "", trim($_REQUEST["we_language_translation"]));
	}
	if((!isset($_SESSION["we_db_collation"]) || empty($_SESSION["we_db_collation"])) && (!isset($_REQUEST["we_db_collation"]) || empty($_REQUEST["we_db_collation"]))){
		$output .= tpl_errorbox("Please select a valid database collation to be  used by webEdition.");
		$errors = true;
	} else if(isset($_REQUEST["we_db_collation"])){
		$_SESSION["we_db_collation"] = str_replace(["/*", '"', "'"], "", trim($_REQUEST["we_db_collation"]));
	}
	$dbcharsetparts = explode('_', $_SESSION["we_db_collation"]);
	$_SESSION["we_db_charset"] = $dbcharsetparts[0];
	$_SESSION['we_charset'] = ($_SESSION["we_db_charset"] == "utf8" ? 'UTF-8' : 'ISO-8859-1');


	// webEdition settings:
	$output .= '<fieldset><legend>webEdition:</legend><table class="small" style="width:100%; table-layout:fixed;">' .
		'<tr><td style="width:160px;">Language*:</td><td>' . (isset($_SESSION["we_language_translation"]) ? htmlentities($_SESSION["we_language_translation"]) : ' - ') . '</td></tr>' .
		'<tr><td>webEdition Code*:</td><td>' . (isset($_SESSION["we_language"]) ? htmlentities($_SESSION["we_language"]) : ' - ') . '</td></tr>' .
		'<tr><td>Default Charset*:</td><td>' . (isset($_SESSION["we_charset"]) ? htmlentities($_SESSION["we_charset"]) : ' - ') . '</td></tr>' .
		'</table></fieldset><br />' .
		// database settings:
		'<fieldset><legend>Database server:</legend><table class="small" style="width:100%; table-layout:fixed;">' .
		'<tr><td style="width:160px;">Server name:</td><td>' . (isset($_SESSION["db_host"]) ? htmlentities($_SESSION["db_host"]) : ' - ') . '</td></tr>' .
		'<tr><td>Database name:</td><td>' . (isset($_SESSION["db_database"]) ? htmlentities($_SESSION["db_database"]) : ' - ') . '</td></tr>' .
		'<tr><td>Table prefix:</td><td>' . (isset($_SESSION["db_tableprefix"]) ? htmlentities($_SESSION["db_tableprefix"]) : ' - ') . '</td></tr>' .
		'<tr><td>Username:</td><td>' . (isset($_SESSION["db_username"]) ? htmlentities($_SESSION["db_username"]) : ' - ') . '</td></tr>' .
		'<tr><td>Password:</td><td>' . (isset($_SESSION["db_password"]) ? htmlentities($_SESSION["db_password"]) : ' - ') . '</td></tr>' .
		'<tr><td>Database charset:</td><td>' . (isset($_SESSION["we_db_charset"]) ? htmlentities($_SESSION["we_db_charset"]) : ' - ') . '</td></tr>' .
		'<tr><td>Database collation:</td><td>' . (isset($_SESSION["we_db_collation"]) ? htmlentities($_SESSION["we_db_collation"]) : ' - ') . '</td></tr>' .
		'<tr><td>DB connection charset*:</td><td>' . (isset($_SESSION["we_db_charset"]) ? htmlentities($_SESSION["we_db_charset"]) : ' - ') . '</td></tr>' .
		'</table></fieldset>';
	if(
		!isset($_SESSION["db_host"]) ||
		empty($_SESSION["db_host"]) ||
		!isset($_SESSION["db_database"]) ||
		empty($_SESSION["db_database"]) ||
		!isset($_SESSION["db_tableprefix"]) ||
		!isset($_SESSION["db_username"]) ||
		empty($_SESSION["db_username"]) ||
		!isset($_SESSION["db_password"]) ||
		!isset($_SESSION["we_language_translation"]) ||
		empty($_SESSION["we_language_translation"]) ||
		!isset($_SESSION["we_language"]) ||
		empty($_SESSION["we_language"])
	){
		$errors = true;
	}
	$output .= "*These values can be changed easily with the configuration dialog.";
	return $output;
}

function step_installation(){
	global $errors;
	$output = "<b>Installation of database tables:</b><br /><br />";

	// read and parse database dump:
	if(!is_readable("./webEdition/liveUpdate/sqldumps") || !is_dir("./webEdition/liveUpdate/sqldumps")){
		$output .= tpl_error("Could not read database dumps directory.");
		$errors = true;
		return $output;
	}

	$sqldumps = scandir('./webEdition/liveUpdate/sqldumps');
	$dbqueries = array();
	foreach($sqldumps as $sqldump){
		if(substr($sqldump, 0, 1) != "." && substr($sqldump, 0, 3) == 'tbl'){
			$dumpcontent = file_get_contents("./webEdition/liveUpdate/sqldumps/" . $sqldump);
			$dumpsarr = explode("/* query separator */", $dumpcontent);
			foreach($dumpsarr as $dbquery){
				$j++;
				$dbqueries[] = $dbquery;
			}
		}
	}

	echo count($dbqueries) . ' queries found.';
	$conn = mysqli_connect($_SESSION["db_host"], $_SESSION["db_username"], $_SESSION["db_password"]);
	if(!$conn){
		$output .= tpl_error("Could not connect to database server. Message from server: " . mysqli_error($conn));
		$errors = true;
		return $output;
	}
	$output .= tpl_ok("connected to database server on \"" . $_SESSION["db_host"] . "\"");

	// select database:
	if(!mysqli_query($conn, "USE " . $_SESSION["db_database"])){
		$output .= tpl_error("Error using specified database. Message from server: " . mysqli_error($conn));
		$errors = true;
		return $output;
	}
	$output .= tpl_ok("Using specified database \"" . $_SESSION["db_database"] . "\"");

	// drop all existing tables beginning with $prefix$tbl:
	$res = mysqli_query($conn, 'show tables where Tables_in_' . $_SESSION["db_database"] . ' LIKE "' . $_SESSION["db_tableprefix"] . 'tbl%"');
	while($table = mysqli_fetch_array($res)){
		mysqli_query($conn, "drop table " . $table[0]);
		echo $table[0] . " dropped.<br />";
	}
	// insert table prefix and install all tables from sql dump:
	$queryErrors = false;

	$res = mysqli_query($conn, 'SHOW variables LIKE "default_storage_engine"');
	$tmp = mysqli_fetch_array($res);
	$defaultEngine = $tmp[1];
	if(!in_array(strtolower($defaultEngine), array('myisam', 'aria'))){
		$defaultEngine = 'myisam';
	}
	$charset_collation = (!empty($_SESSION["we_db_charset"]) && !empty($_SESSION["we_db_collation"]) ?
		" CHARACTER SET " . $_SESSION["we_db_charset"] . " COLLATE " . $_SESSION["we_db_collation"] :
		'') .
		' ENGINE=' . $defaultEngine;
	mysqli_query($conn, " SET NAMES '" . $_SESSION["we_db_charset"] . "' ");
	foreach($dbqueries as $dbquery){
		if(strpos($dbquery, '###UPDATEONLY###') !== false ||
			strpos($dbquery, '###UPDATEDROPCOL') !== false){
			$dbquery = '';
			continue;
		}

		$dbquery = strtr($dbquery, array(
			'###TBLPREFIX###' => (empty($_SESSION["db_tableprefix"]) ? '' : $_SESSION["db_tableprefix"]),
			'###INSTALLONLY###' => '',
			'ENGINE=MyISAM' => $charset_collation
			)
		);

		if(!empty($dbquery)){
			if(!mysqli_query($conn, $dbquery)){
				if(mysqli_errno($conn) != "1065"){
					$output .= tpl_warning("error executing query. Message from server: " . mysqli_error($conn));
					print("<pre>" . $dbquery . "</pre><hr />");
					$queryErrors = true;
				}
				//$output .= tpl_warning("error executing query.");
			} else {
				//print("<pre>".mysql_info($conn)."</pre><hr />");
			}
		}
	}
	$errors |= $queryErrors;

	$output .= ($queryErrors === true ?
		tpl_error("There were some errors while executing the database queries.") :
		tpl_ok("Executed all queries successfully to the selected database.")
		);

	//print("<pre>".$dbdata."</pre>");
	$output .= "<br /><b>Writing webEdition configuration:</b><br /><br />";

	//$output .= "<li><i>under construction ...</i></li>";
	// set the language and backendcharset of the default user
	if(!mysqli_query($conn, 'INSERT INTO ' . $_SESSION['db_tableprefix'] . 'tblPrefs (`userID`,`key`,`value`) VALUES("1","Language","' . $_SESSION["we_language"] . '")') || !mysqli_query($conn, 'INSERT INTO ' . $_SESSION["db_tableprefix"] . 'tblPrefs (`userID`,`key`,`value`) VALUES("1","BackendCharset","' . $_SESSION["we_charset"] . '")')){
		$output .= tpl_warning("Could not change the default user's language settings. Message from server: " . mysqli_error($conn));
		print("<pre>" . $dbquery . "</pre><hr />");
		$queryErrors = true;
		//$output .= tpl_warning("error executing query.");
	} else {
		$output .= tpl_ok("Changed the default user's language to " . $_SESSION["we_language"]);
	}
	mysqli_close($conn);

	//create conf files for productive use
	if(is_writable('./webEdition/we/include/conf/')){
		$dir = './webEdition/we/include/conf/';
		$files = scandir($dir);
		foreach($files as $file){
			if(substr($file, -8) == '.default'){
				copy($dir . $file, $dir . substr($file, 0, -8));
			}
		}
	}

	// write database connection data to we_conf.inc.php
	if(!is_writable('./webEdition/we/include/conf/we_conf.inc.php') || !is_writable('./webEdition/we/include/conf/we_conf_global.inc.php')){
		tpl_error("Could not open webEdition configuration files for writing.");
		$errors = true;
	} else {
		//NOTE these files are only available if version >6.3.1
		$we_config = "<?php\n";
		$we_config_global = "<?php\n";
		$we_active_modules = file_get_contents('./webEdition/we/include/conf/we_active_integrated_modules.inc.php.default');

		//don't worry about undefined symbols, new we versions rewrite missing settings, old versions have if(defined... clauses

		$we_config .= "define('DB_CHARSET','" . $_SESSION["we_db_charset"] . "');\n" .
			"define('DB_COLLATION','" . $_SESSION["we_db_collation"] . "');\n" .
			"define('DB_CONNECT','" . DB_CONNECT . "');\n" .
			"define('DB_HOST','" . $_SESSION["db_host"] . "');\n" .
			"define('DB_DATABASE','" . $_SESSION["db_database"] . "');\n" .
			"define('DB_USER','" . $_SESSION["db_username"] . "');\n" .
			"define('DB_PASSWORD','" . $_SESSION["db_password"] . "');\n" .
			"define('TBL_PREFIX','" . $_SESSION["db_tableprefix"] . "');\n" .
			"define('WE_BACKENDCHARSET','" . $_SESSION["we_charset"] . "');\n" .
			"define('WE_LANGUAGE','" . $_SESSION["we_language"] . "');\n" .
			"";

		//define DB_SET_CHARSET in we_config_global due to old versions, will be corrected on we startup
		$we_config_global .= "define('DB_SET_CHARSET','" . $_SESSION["we_db_charset"] . "');\n" .
			"define('DEFAULT_CHARSET','" . $_SESSION['we_charset'] . "');\n" .
			"";

		$output .= tpl_ok("Changed the system's default language to " . $_SESSION["we_charset"]);
		$output .= tpl_ok("Saved database configuration.");

		if(!(file_put_contents('./webEdition/we/include/conf/we_conf.inc.php', $we_config) && file_put_contents('./webEdition/we/include/conf/we_conf_global.inc.php', $we_config_global) && file_put_contents('./webEdition/we/include/conf/we_active_integrated_modules.inc.php', $we_active_modules) )){
			$output .= tpl_error("Could not write webEdition configuration files.");
			$errors = true;
		} else {
			$output .= tpl_ok("webEdition configuration files written.");
		}
	}
	return $output;
}

function step_finish(){
	return "The webEdition installation is now finished. It is located in the subdirectory \"/webEdition/\", you can enter webEdition by <a href=\"/webEdition/\" target=\"_blank\">clicking here</a>.
	If you want more informations about how to use webEdition, visit our website or join the webEdition community.<br /><br />

<b>Important:</b><br /><br />
Please don't forget to remove this setup script in order to prevent damage to your website by misuse. The next and final step of this installation script will take care of that.<br /><br />
The first thing you should do is to change the default password and username to less obvious ones, by default it is:
	<p style=\"margin-left:20px;\"><b>Username:</b> admin<br /><b>Password:</b> admin</p>
	You can do that using the webEdition user management module (located at the top of the \"Modules\" menu).";
}

function step_cleanup(){
	$filesToDelete = array(
		"." . DIRECTORY_SEPARATOR . "README.txt",
		"." . DIRECTORY_SEPARATOR . "BUILD	",
		"." . DIRECTORY_SEPARATOR . "BUILDDATE",
		"." . DIRECTORY_SEPARATOR . "INSTALL.txt",
		"." . DIRECTORY_SEPARATOR . "LICENSE.txt",
		"." . DIRECTORY_SEPARATOR . "database.sql",
		"." . DIRECTORY_SEPARATOR . "setup.php"
	);
	$error = false;
	foreach($filesToDelete as $fileToDelete){
		if(is_file($fileToDelete)){
			@unlink($fileToDelete);
			if(is_readable($fileToDelete) && $fileToDelete != "." . DIRECTORY_SEPARATOR . "setup.php"){
				$error = true;
			}
		}
	}
	//if(is_readable("./setup.php")) $error = true;
	return "The webEdition installation is now finished. It is located in the subdirectory \"/webEdition/\", you can enter webEdition by <a href=\"/webEdition/\">clicking here</a>.
	If you want more informations about how to use webEdition, visit our website or join the webEdition community.<br /><br />
	" .
		($error === true ?
		tpl_errorbox("At least one of the setup files could not be deleted (maybe insufficient access permissions?), please do that manually!") :
		tpl_infobox("All setup files have been deleted successfully to avoid system damage by misuse.")
		) .
		"<br /><b>Important:</b><br /><br />
The first thing you should do is to change the default password and username to less obvious ones, by default it is:
	<p style=\"margin-left:20px;\"><b>Username:</b> admin<br /><b>Password:</b> admin</p>
	You can do that using the webEdition user management module (located at the top of the \"Extras\" menu).";
}

// html template functions:
// error message box:
function tpl_errorbox($text = ""){
	return '<div style="display:block; padding:3px; padding-left:24px; margin:3px 0px 3px 0px; border:1px solid red; background: url(./webEdition/images/icons/invalid.gif) 3px center no-repeat;" />' . $text . '</div>';
}

// info message box:
function tpl_infobox($text = ""){
	return '<div style="display:block; padding:3px; padding-left:24px; margin:3px 0px 3px 0px; border:1px solid green; background: url(./webEdition/images/icons/valid.gif) 3px center no-repeat;" />' . $text . '</div>';
}

// informational message:
function tpl_info($text = ""){
	return '<p>INFO: ' . $text . '</p>';
}

// error message:
function tpl_error($text = ""){
	return '<li><font color="red">ERROR: </font>' . $text . '</li>';
}

// succes message:
function tpl_ok($text = ""){
	return '<li>' . $text . ' - <font color="green">OK</font></li>';
}

// warning message:
function tpl_warning($text = ""){
	return '<li><font color="orange">WARNING:</font> ' . $text . '</li>';
}

// title text
$stepTitle = (isset($currentStep["title"]) ? '<h1>' . $currentStep["id"] . '. ' . $currentStep["title"] . '</h1><br /><br />' : '');

// step navigation (2 buttons):
function tpl_navigation($step = "1"){
	global $steps, $errors;
	$nextID = $step + 1;
	$prevID = $step - 1;
	// next button

	return '<div style="display:block; margin:10px 0px 10px 0px;"><div style="float:left;"><button style="width:120px;" title="previous step" type="button" onclick="document.location=\'?step=' . $prevID . '\'" ' . ($step == "1" || $step >= (count($steps) - 1) ? 'disabled="disabled"' : '') . '>back</button></div>' .
		'<div style="float:right;"><button style="width:120px;" title="next step" type="button" onclick="document.location=\'?step=' . $nextID . '\'" ' . ($step >= count($steps) || $errors === true ? 'disabled="disabled"' : '') . '>' . ($step == (count($steps) - 1) ? 'cleanup' : 'next') . '</button></div></div>';
}

// buffer
ob_start();
$output = (is_callable('step_' . $currentStep['name']) ?
	call_user_func('step_' . $currentStep['name']) :
	'<br /><i>under construction...</i><br /><br /><br /><br /><br /><br />'
	);

$navigation = tpl_navigation($currentStep['id']);
$bufferedOutput = ob_get_clean();
?>

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
  "http://www.w3.org/TR/html4/loose.dtd">
<html lang="en">
	<head>
		<title>webEdition &bull; initial configuration</title>
		<meta http-equiv="expires" content="0">
		<meta http-equiv="pragma" content="no-cache">
		<meta http-equiv="content-type" content="text/html; charset=UTF-8">
		<meta http-equiv="imagetoolbar" content="no">
		<meta name="generator" content="webEdition Version <?php echo WE_VERSION; ?>">
		<style type="text/css">
			body, div, div.small, table, td, font, li, form {
				font-size:8pt;
			}
			table {font-weight:normal;}
			td {
				text-align: left;
			}
			fieldset {
				border:1px solid #888;
			}
			legend {
				font-weight:bold;
			}
			div.debug {
				position:absolute;
				margin:0px auto;
				background:transparent;
				width:100%;
				height:110px;
				overflow:auto;
				z-index:99;
				font-size:9pt;
				font-weight:normal;
				border-bottom:1px solid #333;
			}
			a {
				color:#000000;
			}
			a:visited {
				color:#000000;
			}
		</style>
		<link href="webEdition/css/webEdition.css" />
	</head>
	<body bgcolor="#386AAB" class="header" style="margin:0px">
		<div class="debug"<?= (isset($_SESSION["debug"]) ? ' style="display:block;"' : ' style="display:none;"'); ?>>
			<?= $bufferedOutput; ?>
		</div>
		<form action="/setup.php?step=<?= ($currentStep["id"] + 1) ?>" method="post">
			<input name="step" value="<?= $currentStep["id"] + 1 ?>" type="hidden" />
			<table border="0" style="margin-left: auto;margin-right: auto;width:818px;">
				<tr style="height:10px;">
					<td style="width:260px;background-color:#386AAB;"></td>
					<td rowspan="2" style="width:430px;">
						<table border="0" style="background-repeat: no-repeat;background-color:#EBEBEB;">
							<tr><td colspan="3" width="432" height="110"><img src="/webEdition/images/webedition.svg"/></td></tr>
							<tr><td width="432" colspan="3"></td></tr>
							<tr>
								<td width="15"></td>
								<td width="402">
									<?=
									$stepTitle .
									$output .
									$navigation;
									?>
								</td>
								<td width="15"></td>
							</tr>
							<tr><td width="432" colspan="3"></td></tr>
							<tr>
								<td width="15"></td>
								<td width="402" class="small">Version: <?php echo WE_VERSION ?></td>
								<td width="15"></td>
							</tr>
						</table>
					</td>
					<td valign="top" style="width:260px;background-repeat:repeat-y;"></td>
				</tr>
				<tr>
					<td valign="bottom" style="width:260px;height:296px;background-color:#386AAB;"></td>
					<td valign="bottom" style="width:260px;height:296px;background-repeat:repeat-y;"></td>
				</tr>
				<tr style="height:10px;">
					<td style="width:260px;"></td>
					<td style="height:10px;"></td>
					<td style="width:260px;"></td>
				</tr>
			</table>
		</form>
	</body>
</html>
