<?php
/**
 * webEdition CMS
 *
 * $Rev: 15243 $
 * $Author: mokraemer $
 * $Date: 2018-05-08 19:44:42 +0200 (Tue, 08 May 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Temporary document
 *
 * @static

 */
abstract class we_contents_temporaryDocument{

	/**
	 * Save document in temporary table
	 *
	 * @static
	 * @access public
	 *
	 * @param int documentID ID for document which will be stored in database
	 * @param object mixed document object
	 */
	static function save($documentID, $table, array $document, we_database_base $db){
		$did = intval($documentID);
		$ret = $db->query('REPLACE INTO ' . TEMPORARY_DOC_TABLE . ' SET ' .
			we_database_base::arraySetter([
				'DocumentID' => $did,
				'DocumentObject' => ($document ? we_serialize($document, SERIALIZE_JSON) : ''),
				'DocTable' => stripTblPrefix($table)
		]));
		return $ret;
	}

	/**
	 * Load document from temporary table
	 *
	 * @static
	 * @access public
	 *
	 * @param int documentID Document ID
	 * @return object mixed document object. if return value is flase, document doesn't exists in temporary table
	 */
	static function load($documentID, $table, we_database_base $db, $keep = false){
		$dat = f('SELECT DocumentObject FROM ' . TEMPORARY_DOC_TABLE . ' WHERE DocumentID=' . intval($documentID) . ' AND DocTable="' . $db->escape(stripTblPrefix($table)) . '"', '', $db);
		return ($keep ? $dat : we_unserialize($dat));
	}

	/**
	 * Delete document from temporary table
	 *
	 * @static
	 * @access public
	 *
	 * @param int documentID Document ID
	 */
	static function delete($documentID, $table, we_database_base $db){
		return $db->query('DELETE FROM ' . TEMPORARY_DOC_TABLE . ' WHERE DocumentID=' . intval($documentID) . ' AND DocTable="' . $db->escape(stripTblPrefix($table)) . '"');
	}

	static function isInTempDB($id, $table, we_database_base $db){
		return (intval($id) > 0 ?
			(f('SELECT 1 FROM ' . TEMPORARY_DOC_TABLE . ' WHERE DocumentID=' . intval($id) . ' AND DocTable="' . $db->escape(stripTblPrefix($table)) . '"  LIMIT 1', '', $db)) :
			false);
	}

}
