<?php

/**
 * webEdition CMS
 *
 * $Rev: 15612 $
 * $Author: mokraemer $
 * $Date: 2018-08-22 18:57:18 +0200 (Wed, 22 Aug 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_htmlDocument extends we_contents_textContentDocument{

	function __construct(){
		parent::__construct();
		$this->ContentType = we_base_ContentTypes::HTML;
	}

	protected function saveContentDataInDB(){
		if(($code = $this->getElement('data'))){
			$metas = $this->getMetas($code);
			if(!empty($metas['title'])){
				$this->setElement(self::TITLE_FIELD, $metas['title']);
			}
			if(!empty($metas['description'])){
				$this->setElement(self::DESCRIPTION_FIELD, $metas['description']);
			}
			if(!empty($metas['keywords'])){
				$this->setElement(self::KEYWORD_FIELD, $metas['keywords']);
			}
			if(!empty($metas['charset'])){
				$this->setElement('Charset', $metas['charset'], 'attrib');
			}
		}
		return parent::saveContentDataInDB();
	}

	function getDocumentCode(){
		$code = $this->getElement('data');
		if(($cs = $this->getElement('Charset'))){
			$code = preg_replace('/<meta http-equiv="Content-Type" content=".*>|<meta charset=".*>/i', we_html_tools::metaCtCharset($cs), $code);
		}
		return $code;
	}

	public function getPropertyPage(we_base_jsCmd $jsCmd){
		return we_html_multiIconBox::getHTML('PropertyPage', [
				['icon' => we_html_multiIconBox::PROP_PATH, 'headline' => g_l('weClass', '[path]'), 'html' => $this->formPath(),],
				['icon' => we_html_multiIconBox::PROP_DOC, 'headline' => g_l('weClass', '[document]'), 'html' => $this->formDocTypeTempl(),],
				(we_modules_info::isActive(we_modules_info::CATEGORY) ? ['icon' => we_html_multiIconBox::PROP_CATEGORIES, 'headline' => g_l('global', '[categorys]'), 'html' => $this->formCategory($jsCmd), 'fold' => true] : null),
				['icon' => we_html_multiIconBox::PROP_NAVI, 'headline' => g_l('global', '[navigation]'), 'html' => $this->formNavigation($jsCmd), 'fold' => true],
				['icon' => we_html_multiIconBox::PROP_COPY, 'headline' => g_l('weClass', '[copy' . $this->ContentType . ']'), 'html' => $this->formCopyDocument(),],
				['icon' => we_html_multiIconBox::PROP_CHARSET, 'headline' => g_l('weClass', '[Charset]'), 'html' => $this->formCharset(), 'fold' => true],
				['icon' => we_html_multiIconBox::PROP_USER, 'headline' => g_l('weClass', '[owners]'), 'html' => $this->formCreatorOwners($jsCmd), 'fold' => true]], true
		);
	}

}
