<?php

/**
 * webEdition CMS
 *
 * $Rev: 15612 $
 * $Author: mokraemer $
 * $Date: 2018-08-22 18:57:18 +0200 (Wed, 22 Aug 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_dialog_wysiwyg extends we_dialog_base{
	protected $isFrontend = false;
	protected $name = '';
	protected $fieldName = '';
	protected $dialogProperties = [];
	protected $editorType = '';

	protected function __construct($noInternals = true, $editorType = we_wysiwyg_editor::TYPE_INLINE_FALSE){
		parent::__construct($noInternals);

		$this->editorType = $editorType;
		if($this->editorType === we_wysiwyg_editor::TYPE_INLINE_FALSE){
			$this->dialogTitle = 'WYSIWYG-Editor';
		} else {
			$this->onOkJsOnly = true;
			$this->dialogTitle = g_l('wysiwyg', '[fullscreen_editor]');
			$this->bodyId = weFullscreenDialog;
		}

		$this->initByHttp();
		$this->isTinyDialog = $this->dialogProperties['weEditorType'] === 'fullscreen';
	}

	protected function initByHttp(){
		parent::initByHttp();

		$this->dialogProperties = (array) we_unserialize($this->args['dialogProperties']);
		$this->charset = $this->dialogProperties['weCharset'];
		if(!$this->charset){
			t_e('charset not found for wysiwyg', $this->charset);
			exit();
		}

		$this->name = $this->dialogProperties['weName'];
		$this->fieldName = $this->dialogProperties['weFieldName'];
		$this->isFrontend = $this->dialogProperties['weIsFrontendEdit'];
	}

	protected function getJs(){
		return parent::getJs() .
			we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'plugins/wefullscreen/js/fullscreen_init.js') .
			we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_wysiwyg.js', 'self.focus();') .
			we_wysiwyg_editor::getHTMLHeader($this->noInternals);
	}

	

	protected function getDialogContentHTML(){
		$e = new we_wysiwyg_editor($this->dialogProperties, $this->dialogProperties['weEditorType']);

		return we_html_element::div(['style' => 'position:absolute;top:0;bottom:0;left:0;right:0;overflow:hidden;margin:0;'], $e->getHTML());
	}

	protected function getDialogHTML(){
		if($this->editorType === we_wysiwyg_editor::TYPE_FULLSCREEN){
			return parent::getDialogHTML();
		}
		$dialogContent = we_html_tools::dialogLayout($this->getDialogContentHTML(), '<i class="fa fa-2x fa-spinner fa-pulse"></i>', $this->getDialogButtons());

		return $this->getFormHTML() . $dialogContent . we_html_element::hidden('we_what', 'cmd') . $this->getHiddenArgs() . '</form>';
	}

	protected function getBodyEndpart(){
		return we_wysiwyg_editor::getHTMLConfigurationsTag();
	}

	protected function cmdFunction(array $args){
		switch($this->we_cmd[0]){
			case 'open_dialog_fullscreen':
				$this->jsCmd->addCmd('fullscreen_writeback');
				return we_html_tools::getHtmlTop($this->fieldName, $this->charset, '', $this->jsCmd->getCmds() . we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_cmdFrame.js'), we_html_element::body()
				);
				break;
			case 'writeBack_inlineFalse':
			case 'open_wysiwyg_window':
				// do we need this twice?
				if(!$this->isFrontend){
					if(preg_match('%^(.+_te?xt)\[.+\]$%i', $this->name)){
						$reqName = preg_replace('/^(.+_te?xt)\[.+\]$/', '${1}', $this->name);
					} else if(preg_match('|^(.+_input)\[.+\]$|i', $this->name)){
						$reqName = preg_replace('/^(.+_input)\[.+\]$/', '${1}', $this->name);
					}
				} else {
					$reqName = str_replace('[' . $this->fieldName . ']', '', $this->name);
				}

				$value = preg_replace('|(</?)script([^>]*>)|i', '${1}scr"+"ipt${2}', strtr(we_base_request::_(we_base_request::RAW_CHECKED, $reqName, '', $this->fieldName), [
					//"\r" => '\r',
					//"\n" => '\n',
					"'" => '&#039;'
				]));
				$replacements = [
					//'"' => '\"',
					"\xe2\x80\xa8" => '',
					"\xe2\x80\xa9" => '',
				];
				$taValue = strtr($value, $replacements);
				$divValue = $this->isFrontend ? $taValue : strtr(we_contents_document::parseInternalLinks($value, 0), $replacements);

				return we_html_tools::getHtmlTop($this->fieldName, $this->charset, '', we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_cmdFrame.js', "we_cmd('wysiwyg_writeBack')", [
							'id' => 'loadVarDialog_cmdFrame',
							'data-payload' => setDynamicVar(['textareaValue' => $taValue,
								'divValue' => $divValue,
								'name' => $this->name,
								'isFrontendEdit' => $this->isFrontend ? 1 : 0]),
							'editorTyoe' => $this->editorType
						]), we_html_element::body()
				);
		}
		return'';
	}

}
