<?php

/**
 * webEdition CMS
 *
 * $Rev: 15394 $
 * $Author: mokraemer $
 * $Date: 2018-06-21 14:06:00 +0200 (Thu, 21 Jun 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* the parent class of storagable webEdition classes */
class we_doctype_view extends we_modules_view{

	function __construct(){
		parent::__construct();
		$this->Model = new we_doctype_docTypes();
	}

	function getJSTop(){
		$modData = we_modules_info::getModuleField(we_modules_info::DOCTYPE, 'text');
		$title = ($modData ? 'webEdition ' . g_l('global', '[modules]') . ' - ' . $modData : '');

		return
			parent::getJSTop() .
			we_html_element::jsScript(WE_JS_MODULES_DIR . '/doctype/doctype_top.js', "parent.document.title='" . $title . "'");
	}

	function getJSProperty(array $jsVars = []){
		return we_html_element::jsScript(WE_JS_MODULES_DIR . '/doctype/doctype_prop.js');
	}

	function processCommands(we_base_jsCmd $jsCmd){
		switch(we_base_request::_(we_base_request::STRING, "we_cmd", '', 0)){
			case 'save_docType':
				if(!we_base_permission::hasPerm(we_base_perm::WORK_DOCTYPE_EDIT)){
					$jsCmd->addMsg(g_l('weClass', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$this->Model->Path = ($this->Model->ParentID ? f('SELECT Path FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . intval($this->Model->ParentID)) : '/') . $this->Model->DocType;
				if(preg_match('|[\'",]|', $this->Model->DocType)){
					$jsCmd->addMsg(g_l('alert', '[doctype_hochkomma]'), we_base_util::WE_MESSAGE_ERROR);
				} else if(!$this->Model->DocType){
					$jsCmd->addMsg(g_l('alert', '[doctype_empty]'), we_base_util::WE_MESSAGE_ERROR);
				} elseif(f('SELECT ID FROM ' . DOC_TYPES_TABLE . ' dt WHERE dt.DocType="' . $GLOBALS['DB_WE']->escape($this->Model->DocType) . '" AND ID!=' . intval($this->Model->ID) . ' LIMIT 1')){
					$jsCmd->addMsg(sprintf(g_l('weClass', '[doctype_save_nok_exist]'), $this->Model->DocType), we_base_util::WE_MESSAGE_ERROR);
				} elseif($this->Model->we_save()){
					$jsCmd->addMsg(sprintf(g_l('weClass', '[doctype_save_ok]'), $this->Model->DocType), we_base_util::WE_MESSAGE_NOTICE);
					list($cmd, $val) = we_main_headermenu::getMenuReloadCode('top.opener.', true);
					$jsCmd->addCmd($cmd, $val);
					if($this->Model->wasUpdate){
						$jsCmd->addCmd('updateTreeEntry', ['id' => $this->Model->ID, 'parentid' => $this->Model->ParentID, 'text' => $this->Model->DocType]);
					} else {
						$jsCmd->addCmd('makeTreeEntry', ['id' => $this->Model->ID, 'parentid' => $this->Model->ParentID, 'text' => $this->Model->DocType, 'open' => false, 'contenttype' => 'we/doctype',
							'table' => DOC_TYPES_TABLE, 'published' => 1]);
					}
				} else {
					t_e('ERROR');
				}
				break;
			case 'newDocType':
				$tmp = $this->Model->Name; // FIXME: find better solution for #11586
				$this->Model = new we_doctype_docTypes();
				$this->Model->Name = $tmp;
				break;
			case 'deleteDocType':
			case 'deleteDocTypeok':
				if(!we_base_permission::hasPerm(we_base_perm::WORK_DOCTYPE_EDIT)){
					$jsCmd->addMsg(g_l('alert', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$id = we_base_request::_(we_base_request::INT, 'we_cmd', 0, 1) ?: $this->Model->ID; // taking id from request may be obsolete
				$name = f('SELECT DocType FROM ' . DOC_TYPES_TABLE . ' dt WHERE dt.ID=' . $id);
				$del = false;
				if($name){
					if(f('SELECT 1 FROM ' . FILE_TABLE . ' WHERE DocType=' . $id . ' LIMIT 1')){
						$jsCmd->addMsg(sprintf(g_l('weClass', '[doctype_delete_nok]'), $name), we_base_util::WE_MESSAGE_ERROR);
					} else {
						$GLOBALS['DB_WE']->query('DELETE FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . $id);
						$GLOBALS['DB_WE']->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="tblDocTypes" AND (DID=' . $id . ' OR LDID=' . $id . ')');

						$jsCmd->addMsg(sprintf(g_l('weClass', '[doctype_delete_ok]'), $name), we_base_util::WE_MESSAGE_NOTICE);
						$jsCmd->addCmd('deleteTreeEntry', $id);
						$jsCmd->addCmd('loadHome');
						$_SESSION['weS']['doctype_session'] = [];
					}
				}
				break;
			case 'add_dt_template':
				$foo = array_merge(explode(',', $this->Model->Templates), we_base_request::_(we_base_request::INTLISTA, 'we_cmd', [], 1));
				$this->Model->Templates = implode(',', array_unique($foo));
				break;
			case 'delete_dt_template':
				$foo = explode(',', $this->Model->Templates);
				$cmd1 = we_base_request::_(we_base_request::INT, 'we_cmd', 0, 1);
				if($cmd1 && ($pos = array_search($cmd1, $foo, false)) !== false){
					unset($foo[$pos]);
				}
				if($this->Model->TemplateID == $cmd1){
					$this->Model->TemplateID = ($foo ? $foo[0] : 0);
				}
				$this->Model->Templates = implode(',', $foo);
				break;
			case 'dt_add_cat':
				if(($id = we_base_request::_(we_base_request::INTLISTA, 'we_cmd', [], 1))){
					$this->Model->addCat($id);
				}
				break;
			case 'dt_delete_cat':
				if(($id = we_base_request::_(we_base_request::INT, 'we_cmd', 0, 1))){
					$this->Model->delCat($id);
				}
				break;
			default:
				$id = (we_base_request::_(we_base_request::INT, 'we_cmd', 0, 1) ?: 0);
				if(!$id){
					$dtq = we_doctype_docTypes::getDoctypeQuery($GLOBALS['DB_WE']);
					$id = f('SELECT dt.ID FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE ' . $dtq['where'] . ' LIMIT 1');
				}

				if($id){
					$this->Model->initByID($id, DOC_TYPES_TABLE);
				}
		}
		$this->Model->saveInSession($_SESSION['weS']['doctype_session']);
	}

	function processVariables(){
		if(!empty($_SESSION['weS']['doctype_session'])){
			$this->Model->we_initSessDat($_SESSION['weS']['doctype_session']);
		}
	}

	public function getHomeScreen(){
		$hiddens = ['cmd' => "home", 'pnt' => 'edbody'];
		$content = we_html_button::create_button('new_doctype', "javascript:we_cmd('newDocType')");

		return parent::getActualHomeScreen("doctype", $content, we_html_element::form(['name' => 'we_form'], $this->getCommonHiddens($hiddens) . we_html_element::hidden("home", 0)));
	}

}
