<?php

/**
 * webEdition CMS
 *
 * $Rev: 15795 $
 * $Author: mokraemer $
 * $Date: 2018-09-21 16:28:50 +0200 (Fri, 21 Sep 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_gui_menu{

	private $entries;

	const E_NEW = 'new';
	const E_FILE = 'file';
	const E_COCKPIT = 'cockpit';
	const E_MODULE = 'module';
	const E_SETTINGS = 'extras';
	const E_HELP = 'help';

	public function __construct($entries){
		if($entries){
			$this->entries = $entries;
		}
	}

	public function getCode(){
		return self::getJS() . $this->getHTML();
	}

	public function getHTML(){
		$out = '<ul id="nav">';
		$menus = [];
		foreach($this->entries as $id => $e){
			if(empty($e['parent'])){
				if(empty($e['perm']) ? 1 : self::isEnabled($e['perm'])){
					$mtext = ($e['text'] ?: '');

					$menus[] = ['id' => $id,
						'code' => '<li class="top" onmouseover="topMenuHover(this)"><div class="top_div" onclick="topMenuClick(this)"><a href="#void" class="top_link">' . we_html_element::span([
							'class' => 'down'], '<i class="icon ' . (empty($e['icon']) ? '' : $e['icon']) . '" title="' . $mtext . '"></i>' . we_html_element::span(['class' => 'mtext'], $mtext)) . '</a><ul class="sub">',
					];
				}
			}
		}

		foreach($menus as $menu){
			$out .= $menu['code'] .
					$this->h_pCODE($this->entries, $menu['id'], '') .
					'</ul></div></li>';
		}

		$out .= '</ul>';
		return $out;
	}

	private static function h_search($men, $p){
		$container = [];
		foreach($men as $id => $e){
			if(isset($e['parent']) && $e['parent'] == $p){
				$container[$id] = $e;
			}
		}
		return $container;
	}

	public static function isEnabled(array $perm){
		if(!$perm){
			return true;
		}
		foreach($perm as $v){
			if(we_base_permission::hasPerm($v)){
				return true;
			}
		}
		return false;
	}

	private function h_pCODE($men, $p, $zweig){
		$nf = self::h_search($men, $p);
		if(empty($nf)){
			return '';
		}

		$opt = '';

		foreach($nf as $id => $e){
			$newAst = $zweig;
			$mtext = (isset($e['text']) && is_array($e['text']) ?
					($e['text'][$GLOBALS['WE_LANGUAGE']] ?: '') :
					(isset($e['text']) ? $e['text'] : ''));

			if(!empty($e['hide']) ||
					(!empty($e['perm']) && !self::isEnabled($e['perm']))
			){
				continue;
			}

			if((!(isset($e['cmd']) && $e['cmd'])) && $mtext){
				$opt .= '<li><a class="fly" href="#void">' . $mtext . '<i class="fa fa-caret-right"></i></a><ul class="menu_' . $id . '">' .
						$this->h_pCODE($men, $id, $newAst) .
						'</ul></li>';
			} else if($mtext){
				$opt .= '<li><a href="#void" onclick="we_cmd(\'' . (is_array($e['cmd']) ? implode('\',\'', $e['cmd']) : $e['cmd']) . '\')">' . $mtext . '</a></li>';
			} else {//separator
				$opt .= '<li class="disabled"></li>';
			}
		}
		return $opt;
	}

}
