<?php

/**
 * webEdition CMS
 *
 * $Rev: 15830 $
 * $Author: mokraemer $
 * $Date: 2018-09-27 15:50:42 +0200 (Thu, 27 Sep 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_import_csvBase{
	protected $CSVFile;
	protected $CSVData = '';
	protected $CSVError = [];
	protected $FieldNames;
	var $Fields;
	var $FieldTypes;
	var $fetchCursor;
	var $Filter = [];

	function setFile($file){
		$this->CSVFile = $file;
		if(file_exists($this->CSVFile) && ($this->CSVFile != "none") && !empty($this->CSVFile)){
			$this->setData(implode('', file($this->CSVFile)));
			return true;
		}
		$this->CSVError[] = "The file " . $file . " does not exist or is empty.";
		return false;
	}

	function setData($string){
		$this->CSVData = $string;
		$this->Fields = [];
		$this->FieldTypes = [];
		$this->FieldNames = [];
	}

	function CSVFetchRow(){
		if($this->fetchCursor <= $this->CSVNumRows()){
			$r = $this->Fields[$this->fetchCursor];
			$this->fetchCursor++;
			return $r;
		}
		$this->CSVError[] = "No more data sets.";
		return false;
	}

	function CSVFetchArray($resultTyp = "BOTH"){
		if($this->fetchCursor <= ($this->CSVNumRows()) - 1){

			switch($resultTyp){
				case 'NUM':
				case 'BOTH':
					$r = $this->CSVFetchRow();
					if($resultTyp === "NUM"){
						return $r;
					}

					$this->fetchCursor--;
			}

			switch($resultTyp){
				case 'ASSOC':
				case 'BOTH':
					if(is_array($this->Fields[$this->fetchCursor])){
						foreach($this->Fields[$this->fetchCursor] as $field_id => $field){
							$r[$this->FieldNames[$field_id]] = $field;
						}
					}
			}
			$this->fetchCursor++;
			return $r;
		}
		$this->CSVError[] = "No more data sets.";
		return false;
	}

	function CSVFetchFieldNames(){
		return $this->FieldNames;
	}

	function CSVFieldName($field_id){
		return $this->FieldNames[$field_id];
	}

	function CSVNumRows(){
		return count($this->Fields);
	}

	function CSVNumFields(){
		return count($this->FieldNames);
	}

	function setCursor($pos){
		$this->fetchCursor = $pos;
	}

	function getCursor(){
		return $this->fetchCursor;
	}

	function resetCursor(){
		$this->setCursor(0);
	}

	function getFieldID($search_field){
		if(!is_array($this->FieldNames)){
			return false;
		}
		foreach($this->FieldNames as $field_id => $field_name){

			if(trim($search_field) == trim($field_name)){
				return $field_id;
			}
		}
		return false;
	}

	function echoCSVError(){
		foreach($this->CSVError as $pos => $error_str){
			echo "- " . ($pos + 1) . ". " . $error_str . "<br/>";
		}
	}

	function isOK($error_output = true){
		if($error_output){
			$this->echoCSVError();
		}
		return empty($this->CSVError);
	}

	function array_merge_better($a1, $a2){
		if(!is_array($a1)){
			$a1 = [];
		}
		if(!is_array($a2)){
			$a2 = [];
		}


		$newarray = $a1;

		foreach($a2 as $key => $val){
			if(is_array($val) && is_array($newarray[$key])){
				$newarray[$key] = $this->array_merge_better($newarray[$key], $val);
			} else {
				$newarray[$key] = $val;
			}
		}
		return $newarray;
	}

	function generateIniValue($array, $filename){
		$val = '';
		foreach($array as $key => $val){
			$val .= "$key = $val;\n";
		}
		we_base_file::save($filename, $val, 'w');
	}

}
