<?php
/**
 * webEdition CMS
 *
 * $Rev: 15766 $
 * $Author: mokraemer $
 * $Date: 2018-09-17 18:44:17 +0200 (Mon, 17 Sep 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class    we_listview_shopOrderitem
 * @desc    class for tag <we:listview type="orderitem">
 *
 */
class we_listview_shopOrderitem extends we_listview_base{
	var $Path = '';
	var $docID = 0;
	var $orderID = 0;
	var $hidedirindex = false;

	const replArray = [
		'OrderID' => 'orderID',
		'OrderIntID' => 'orderID',
		'IntOrderID' => 'orderID',
		'ArticleIntID' => 'orderDocID',
		'IntArticleID' => 'orderDocID', //prevents accidential replacements
		'ArticleID' => 'orderDocID',
		'IntQuantity' => 'quantity', //prevents accidential replacements
		'Quantity' => 'quantity',
	];

	/**
	 * @desc    constructor of class
	 *
	 * @param   $name          string - name of listview
	 * @param   $rows          integer - number of rows to display per page
	 * @param   $order         string - field name(s) to ORDER BY
	 * @param   $desc		   string - if desc order
	 * @param   $condition	   string - condition of listview
	 * @param   $cols		   string - number of cols (default = 1)
	 * @param   $docID	   	   string - id of a document where a we:customer tag is on
	 *
	 */
	function __construct($name, $rows, $offset, $order, $desc, $condition, $cols, $docID, $orderID, $hidedirindex){
		parent::__construct($name, $rows, $offset, $order, $desc, '', false, [], $cols);
		if($GLOBALS['we']['main']['doc']->InWebEdition){
			//do nothing inside we
			return;
		}
		$this->docID = $docID;
		$this->orderID = $orderID;
		$this->condition = $condition;

		$this->Path = ($this->docID ? id_to_path($this->docID, FILE_TABLE, $this->DB_WE) : (isset($GLOBALS['we_doc']) ? $GLOBALS['we_doc']->Path : ''));

		$this->hidedirindex = $hidedirindex;
		// IMPORTANT for seeMode !!!! #5317
		$this->LastDocPath = (isset($_SESSION['weS']['last_webEdition_document']) ? $_SESSION['weS']['last_webEdition_document']['Path'] : '');
		$this->condition = strtr($this->condition, self::replArray);
		$this->order = strtr($this->order, self::replArray);

		$orderstring = ' ORDER BY ' .
			($this->order ?
			$this->order . ' ' . ($this->desc && (!preg_match('|.+ desc$|i', $this->order)) ? ' DESC' : '') :
			'ID' . ($this->desc ? ' DESC' : '')
			);

		$where = ($this->condition ?
			' WHERE ' . $this->condition . ($this->orderID ? ' AND orderID=' . $this->orderID : '') :
			($this->orderID ? ' WHERE orderID=' . $this->orderID : ''));

		$this->anz_all = f('SELECT COUNT(1) FROM ' . SHOP_ORDER_ITEM_TABLE . $where, '', $this->DB_WE);

		$this->DB_WE->query('SELECT
	oi.orderID AS ID,
	oi.orderID AS OrderID,
	oi.orderID AS ' . self::TEXT_FIELD . ',
	oi.orderID AS ' . self::ID_FIELD . ',
	oi.orderDocID,
	oi.quantity AS Quantity,
	oi.Price,
	oi.Vat,
	oi.Vat AS ' . we_shop_shop::VAT_FIELD_NAME . ',
	oi.customFields,
	od.type AS documentType,
	od.SerializedData,
	od.variant AS VARIANT
FROM ' . SHOP_ORDER_ITEM_TABLE . ' oi JOIN
	' . SHOP_ORDER_DOCUMENT_TABLE . ' od ON oi.orderDocID=od.ID ' .
			$where . ' ' .
			$orderstring . ' ' .
			(($this->maxItemsPerPage > 0) ? (' LIMIT ' . $this->start . ',' . $this->maxItemsPerPage) : '')
		);
		$this->anz = $this->DB_WE->num_rows();
	}

	public function next_record(){
		if(($ret = $this->DB_WE->next_record(MYSQLI_ASSOC))){
			$serialDoc = we_unserialize($this->DB_WE->f('SerializedData'));
			$custFields = we_unserialize($this->DB_WE->f('customFields'));

			unset($serialDoc['Charset']);
			$this->Record = array_change_key_case(array_merge($this->DB_WE->getRecord(), $serialDoc, $custFields));
			unset($this->Record['SerializedData'], $this->Record['customFields']);
			$this->Record[self::PATH_FIELD] = $this->Path . '?we_orderid=' . $this->Record['orderid'] . '&we_orderitemid=' . $this->Record['id'];
			$this->Record[self::LASTPATH_FIELD] = $this->LastDocPath . '?we_orderid=' . $this->Record['orderid'] . '&we_orderitemid=' . $this->Record['id'];
			$this->count++;
			return true;
		}

		$this->stop_next_row = $this->shouldPrintEndTR();
		if($this->cols && ($this->count <= $this->maxItemsPerPage) && !$this->stop_next_row){
			$this->Record = [];
			$this->count++;
			return true;
		}

		return false;
	}

	function f($key, $stripWE = false){
		return parent::f($key, $stripWE);
	}

}
