<?php

/**
 * webEdition CMS
 *
 * collection of the navigation items
 *
 * $Rev: 15735 $
 * $Author: mokraemer $
 * $Date: 2018-09-10 12:02:00 +0200 (Mon, 10 Sep 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 *
 * @license		http://www.gnu.org/copyleft/gpl.html  GPL
 * @version    	SVN: $Id$
 */
class we_navigation_items{
	const TEMPLATE_DEFAULT_CURRENT = 'current';
	const TEMPLATE_DEFAULT_POSITION = 'position';
	const TEMPLATE_DEFAULT_LEVEL = 'level';
	const TYPE_ROOT = 'root';
	const TYPE_FOLDER = 'folder';
	const TYPE_ITEM = 'item';

	private static $cache = [];
	public $depth = false;
	var $items = [];
	private $templates = [
		self::TYPE_ROOT => [
			self::TEMPLATE_DEFAULT_LEVEL => [
				self::TEMPLATE_DEFAULT_CURRENT => [
					self::TEMPLATE_DEFAULT_POSITION => '<?= we_tag(\'navigationEntries\'); ?>'
				]]],
		self::TYPE_FOLDER => [
			self::TEMPLATE_DEFAULT_LEVEL => [
				self::TEMPLATE_DEFAULT_CURRENT => [
					self::TEMPLATE_DEFAULT_POSITION => '<li><a href="<?= we_tag(\'navigationField\', [\'name\' => \'href\']); ?>"><?= we_tag(\'navigationField\', [\'name\' => \'text\']); ?></a><?php if(we_tag(\'ifHasEntries\')){ ?><ul><?= we_tag(\'navigationEntries\'); ?></ul><?php } ?></li>'
				]]],
		self::TYPE_ITEM => [
			self::TEMPLATE_DEFAULT_LEVEL => [
				self::TEMPLATE_DEFAULT_CURRENT => [
					self::TEMPLATE_DEFAULT_POSITION => '<li><a href="<?= we_tag(\'navigationField\', [\'name\' => \'href\']); ?>"><?= we_tag(\'navigationField\', [\'name\' => \'text\']); ?></a></li>'
				]]],
	];
	private $rootItem = 0;
	private $hasCurrent = false;
	private $currentRules = [];
	public $weNavigationItemArray = [];
	//this is set for having different functionnames in case the navigation is included more than once (with different templates)
	public $funcPrefix = '';
	private static $Storage = [
		'items' => [],
		'ids' => [0 => '/'],
	];

	static function getCustomerData(we_navigation_navigation $navi){
		//FIXME: check if we need this csv/unserialize code any more
		if(!is_array($navi->Customers)){
			$navi->Customers = array_filter(explode(',', $navi->Customers));
		}

		if(!is_array($navi->BlackList)){
			$navi->BlackList = array_filter(explode(',', $navi->BlackList));
		}

		if(!is_array($navi->WhiteList)){
			$navi->WhiteList = array_filter(explode(',', $navi->WhiteList));
		}

		if(!is_array($navi->CustomerFilter)){
			$navi->CustomerFilter = we_unserialize($navi->CustomerFilter);
		}

		return ($navi->FilterMode != we_customer_abstractFilter::OFF ?
			['id' => $navi->FilterMode == we_customer_abstractFilter::SPECIFIC ? $navi->Customers : [],
			'filter' => $navi->FilterMode == we_customer_abstractFilter::FILTER ? $navi->CustomerFilter : [],
			'blacklist' => $navi->FilterMode == we_customer_abstractFilter::FILTER ? $navi->BlackList : [],
			'whitelist' => $navi->FilterMode == we_customer_abstractFilter::FILTER ? $navi->WhiteList : [],
			'usedocumentfilter' => $navi->UseDocumentFilter ? 1 : 0
			] :
			['id' => [],
			'filter' => [],
			'blacklist' => [],
			'whitelist' => [],
			'usedocumentfilter' => 1
		]);
	}

	private function initRulesFromDB(){
		$this->currentRules = we_navigation_ruleControl::getAllNavigationRules();
		we_navigation_cache::saveRules($this->currentRules);
	}

	private function loopAllRules(){
		if(!$this->hasCurrent){
			$this->checkCurrent();
		}
	}

	public function init($parentid = 0, $showRoot = true){
		if(!$this->initFromCache($parentid, $showRoot)){
			//make sure we use cache next time!
			$this->initById($parentid, $showRoot);
			we_navigation_cache::saveCacheNavigation($parentid, $this->items);
		}
	}

	private function initFromCache($parentid = 0, $showRoot = true){
		$this->rootItem = $parentid;

		if(isset(self::$cache[$parentid])){
			$this->items = self::$cache[$parentid];
		} elseif(($this->items = we_navigation_cache::getCacheFromFile($parentid)) === false){
			$this->items = [];
			return false;
		} else {
			self::$cache[$parentid] = $this->items;
		}

		if(!empty($this->items['id' . $parentid]) && is_object($this->items['id' . $parentid])){
			$this->items['id' . $parentid]->type = !$showRoot || $parentid == 0 ? self::TYPE_ROOT : self::TYPE_FOLDER;
		}
		$this->currentRules = we_navigation_cache::getCachedRule();
		if($this->currentRules === false){
			$this->currentRules = [];
			$this->initRulesFromDB();
		}

		foreach($this->items as $item){
			if(is_object($item) && method_exists($item, 'isCurrent')){
				$this->hasCurrent |= ($item->isCurrent($this));
			}
		}
		unset($item);
		$this->loopAllRules();
		return true;
	}

	private function initById($parentid = 0, $showRoot = true){
		$this->items = [];
		$this->rootItem = intval($parentid);
		$navigation = new we_navigation_navigation();
		$this->readItemsFromDb($this->rootItem);
		$item = self::getItemFromPool($parentid) ?: ['ID' => 0, 'Path' => '/'];
		$navigation->initByRawData($item);

		list($table, $linkid) = $navigation->getTableIdForItem();
		$this->items['id' . $navigation->ID] = new we_navigation_item($navigation->ID, $linkid, $table, $navigation->Text, $navigation->Display, $navigation->getHref(), (!$showRoot || $navigation->ID == 0 ? self::TYPE_ROOT : ($navigation->IsFolder ? self::TYPE_FOLDER : self::TYPE_ITEM)), self::id2path($navigation->IconID), $navigation->Attributes, intval($navigation->FilterMode), self::getCustomerData($navigation), $navigation->CurrentOnUrlPar, $navigation->CurrentOnAnker);

		$items = we_navigation_navigation::getDynamicPreview($navigation->ID, self::$Storage['items'], true);
//reduce Memory consumption!
		self::$Storage['items'] = [];

		foreach($items as $item){
			if(!$item['id']){
				continue;
			}
			if(!empty($item['name'])){
				$item['text'] = $item['name'];
			}
			$this->items['id' . $item['id']] = new we_navigation_item($item['id'], $item['docid'], $item['table'], $item['text'], $item['display'], $item['href'], $item['type'], $item['icon'], $item['attributes'], $item['filtermode'], $item['customers'], $item['currentonurlpar'], $item['currentonanker'], $item['currentoncat'], $item['catparam']);

			if(isset($this->items['id' . $item['parentid']])){
				$this->items['id' . $item['parentid']]->addItem($this->items['id' . $item['id']]);
			}

			$this->hasCurrent |= ($this->items['id' . $item['id']]->isCurrent($this));

// add currentRules
			if(isset($item['currentRule'])){
				$this->currentRules[] = $item['currentRule'];
			}
		}

		$this->initRulesFromDB();
		$this->loopAllRules();

//make avail in cache
		self::$cache[$parentid] = $this->items;
	}

	private function checkCategories(array $idsRule, array $idDoc){
		if(!$idsRule){
			return true;
		}
		$diff = array_intersect($idDoc, $idsRule);
		return !empty($diff);
	}

	function setCurrent($navigationID){
		if(isset($this->items['id' . $navigationID])){
			$this->items['id' . $navigationID]->setCurrent($this);
		}
	}

	private function checkCurrent(){
		if(empty($GLOBALS['we']['main']['doc'])){
			return false;
		}

		$candidate = 0;
		$score = 3;
		$pathLen = 0;

		$isObject = (isset($GLOBALS['we_obj']) && !$GLOBALS['we']['main']['doc']->IsFolder);
		$mainCats = array_filter(explode(',', $GLOBALS['we']['main']['doc']->Category));

		$currentWorkspace = $isObject ? //webEdition object
			(defined('WE_REDIRECTED_SEO') ? //webEdition object uses SEO-URL
			substr(WE_REDIRECTED_SEO, 0, strripos(WE_REDIRECTED_SEO, $GLOBALS['we_obj']->Url)) :
			parse_url(urldecode($_SERVER['REQUEST_URI']), PHP_URL_PATH)
			) : //webEdition document
			$GLOBALS['we']['main']['doc']->Path;

		foreach($this->currentRules as $rule){
			$ponder = 4;
			$rulePath = '';

			if(($cats = array_filter(explode(',', $rule->Categories)))){
				if(!$this->checkCategories($cats, $mainCats)){
					continue; // remove from selection
				}
				$ponder--;
			}

			switch($rule->SelectionType){
				case we_navigation_navigation::DYN_DOCTYPE:
					if($isObject){
						continue; // remove from selection
					}
					if($rule->DoctypeID){
						if(empty($GLOBALS['we']['main']['doc']->DocType) || ($rule->DoctypeID != $GLOBALS['we']['main']['doc']->DocType)){
							continue;
						}
						$ponder--;
					}

					if($rule->FolderID){
						$rulePath = rtrim(self::id2path($rule->FolderID), '/') . '/';
					}
					break;
				case we_navigation_navigation::DYN_CLASS:
					if(!$isObject){
						continue; // remove from selection
					}
					if($rule->ClassID){
						if($GLOBALS['we_obj']->TableID != $rule->ClassID){
							continue; // remove from selection
						}
						$ponder--;
					}

					if($rule->WorkspaceID){
						$rulePath = rtrim(self::id2path($rule->WorkspaceID), '/') . '/';
					}
					break;
			}

			if(!empty($rulePath) && strpos($currentWorkspace, $rulePath) !== false){
				$ponder--;
				if(($currPathLen = strlen($rulePath)) >= $pathLen){ //the longest path wins
					if($pathLen > 0){//no ponder for first match
						$ponder--;
					}
					$pathLen = $currPathLen;
				}
			}

			if($ponder <= $score){
				if(NAVIGATION_RULES_CONTINUE_AFTER_FIRST_MATCH){
					$this->setCurrent($rule->NavigationID);
				} else {
					$score = $ponder;
					$candidate = $rule->NavigationID;
				}
			}
		}

		if($candidate){
			$this->setCurrent($candidate);
			return true;
		}

		return false;
	}

	function getItemIds($id){
		$items = [$id];

		foreach($this->items[$id]->items as $key => $val){
			if($val->type == self::TYPE_FOLDER){
				$items = array_merge($items, $this->getItemIds($key));
			} else {
				$items[] = $key;
			}
		}

		return $items;
	}

	function getItems($id = false){
		return ($id ?
			$this->getItemIds($id) :
			array_keys($this->items));
	}

	function getItem($id){
		return isset($this->items[$id]) ? $this->items[$id] : false;
	}

	/**
	 * @param we_navigation_item $item
	 * @return mixed
	 */
	function getTemplate(we_navigation_item $item){
		$currentPos = we_navigation_item::$currentPosition[$item->level];
// get correct Level
		$level = (isset($this->templates[$item->type][$item->level]) ? $item->level : self::TEMPLATE_DEFAULT_LEVEL);
		$useTemplate = $this->templates[$item->type][$level];
		$fkt = $item->type . '_' . $level;
// use current true/false
		if(isset($useTemplate[$item->current])){
			$useTemplate = $useTemplate[$item->current];
			$fkt .= '_' . ($item->current ? '1' : '0');
		} elseif(isset($useTemplate[self::TEMPLATE_DEFAULT_CURRENT])){
			$useTemplate = $useTemplate[self::TEMPLATE_DEFAULT_CURRENT];
			$fkt .= '_' . self::TEMPLATE_DEFAULT_CURRENT;
		} else {
			return $this->getDefaultTemplate($item->type);
		}
//get position
// is last entry??
		if(isset($useTemplate['last']) &&
// check if item is last
			($this->items['id' . $item->parentid]->getLastVisibleItemID() == $item->id)){
			return $fkt . '_last';
		}

		if(isset($useTemplate[$currentPos])){
			return $fkt . '_' . $currentPos;
		}

		if(isset($useTemplate['odd']) && $currentPos % 2 === 1){
			return $fkt . '_odd';
		}

		if(isset($useTemplate['even']) && $currentPos % 2 === 0){
			return $fkt . '_even';
		}

		if(isset($useTemplate[self::TEMPLATE_DEFAULT_POSITION])){
			return $fkt . '_' . self::TEMPLATE_DEFAULT_POSITION;
		}

		return $this->getDefaultTemplate($item->type);
	}

	private function getDefaultTemplate($type){
		return $type . '_' . self::TEMPLATE_DEFAULT_LEVEL . '_' . self::TEMPLATE_DEFAULT_CURRENT . '_' . self::TEMPLATE_DEFAULT_POSITION;
	}

	private static function getTemplateCode(array $templates, $prefix = ''){
		$ret = '';
		foreach($templates as $key => $cur){
			$key = ($prefix ? $prefix . '_' : '') . $key;
			if(is_array($cur)){
				$ret .= self::getTemplateCode($cur, $key);
			} else {
				$ret .= '
public static function ' . $key . '(){
	?>' . $cur . '<?php
}
';
			}
		}
		return $ret;
	}

	public function writeNavigation($depth = false){
		$this->depth = $depth;
		$this->funcPrefix = 'nav_' . sprintf('%u', crc32(serialize($this->templates)));
		$filename = $this->funcPrefix . '.php';
		$file = TEMP_PATH . $filename;
		if(!file_exists($file)){
			$code = self::getTemplateCode($this->templates);
			$file = we_base_file::saveTemp('<?php class ' . $this->funcPrefix . '{' . $code . '}', $filename, 24 * 3600);
		}
		include_once($file);
		if(isset($this->items['id' . $this->rootItem]) && ($this->items['id' . $this->rootItem] instanceof we_navigation_item)){
			if($this->items['id' . $this->rootItem]->type === self::TYPE_FOLDER && $depth !== false){
// if initialised by id => root item is on lvl0 -> therefore decrease depth
// this is to make it equal init by id, parentid
				$depth--;
			}
			we_navigation_item::$currentPosition = [];
			ob_start();
			$this->items['id' . $this->rootItem]->writeItem($this, $depth);
			return ob_get_clean();
		}

		return '';
	}

	public function setTemplate($content, $type, $level, $current, $position){
		$pos = ($position === 'first' ? 1 : $position);
		$this->templates[$type][$level][$current][$pos] = $content;
	}

	private function readItemsFromDb($id){
		$db = $GLOBALS['DB_WE'];
		$ids = [];

		self::$Storage['items'] = $db->getAllFirstq('SELECT n.ID AS i,n.* FROM ' . NAVIGATION_TABLE . ' n ' . ($id ? ' WHERE n.Path LIKE(SELECT CONCAT(nn.Path,"/%") FROM ' . NAVIGATION_TABLE . ' nn WHERE nn.ID=' . intval($id) . ') OR n.ID=' . intval($id) : '') . ' ORDER BY n.Ordn', true, MYSQLI_ASSOC);

		foreach(self::$Storage['items'] as &$tmpItem){
			$tmpItem['Name'] = $tmpItem['Text'];

			switch($tmpItem['SelectionType']){
				case we_navigation_navigation::STYPE_DOCLINK:
					if($tmpItem['Selection'] === we_navigation_navigation::SELECTION_STATIC){
						$ids[] = $tmpItem['LinkID'];
					}
					break;
				case we_navigation_navigation::STYPE_CATLINK:
					if($tmpItem['LinkSelection'] !== we_navigation_navigation::LSELECTION_EXTERN){
						$ids[] = $tmpItem['UrlID'];
					}
					break;
			}

			if($tmpItem['DynamicSelection'] === we_navigation_navigation::DYN_CATEGORY && $tmpItem['LinkSelection'] !== we_navigation_navigation::LSELECTION_EXTERN){
				$ids[] = $tmpItem['UrlID'];
			}

			if($tmpItem['IconID']){
				$ids[] = $tmpItem['IconID'];
			}
		}
		$ids = $ids ? array_diff(array_unique($ids), array_keys(self::$Storage['ids'])) : [];
		if($ids){
			//keep array index
			self::$Storage['ids'] = self::$Storage['ids'] + $db->getAllFirstq('SELECT ID,IF(Published>0,Path,"") FROM ' . FILE_TABLE . ' WHERE ID IN(' . implode(',', $ids) . ') ORDER BY ID', false);
		}
	}

	private static function getItemFromPool($id){
		if(isset(self::$Storage['items'][$id])){
			return self::$Storage['items'][$id];
		}
		return null;
	}

	public static function id2path($id){
		if(isset(self::$Storage['ids'][$id])){
			return self::$Storage['ids'][$id];
		}
		if(!$id){
			return '/';
		}
		return (self::$Storage['ids'][$id] = id_to_path($id, FILE_TABLE));
	}

}
