<?php
/**
 * webEdition CMS
 *
 * $Rev: 15583 $
 * $Author: mokraemer $
 * $Date: 2018-08-17 23:22:45 +0200 (Fri, 17 Aug 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Definition of WebEdition Newsletter Block
 *
 */
class we_newsletter_group extends we_newsletter_base{
// properties start
	var $ID = 0;
	var $NewsletterID = 0;
	var $Emails = '';
	var $Extern;
	var $Customers = '';
	var $SendAll = 1;
	public $Filter = ''; //due to processVariables
	var $settings;
// properties end

	private $aFilter = [];

	/*	 * *****************************************************
	 * Default Constructor
	 * Can load or create new Newsletter Group depends of parameter
	 * ****************************************************** */

	public function __construct($groupID = 0){
		parent::__construct();
		$this->table = NEWSLETTER_GROUP_TABLE;

		$this->persistents = [
			'NewsletterID' => we_base_request::INT,
			'Emails' => we_base_request::RAW_CHECKED,
			'Extern' => we_base_request::WEFILELIST,
			'Customers' => we_base_request::RAW,
			'SendAll' => we_base_request::BOOL,
			'Filter' => we_base_request::SERIALIZED_KEEP
		];

		$this->settings = self::getSettings();
		$this->Extern = isset($this->settings['global_mailing_list']) ? $this->settings['global_mailing_list'] : '';

		if($groupID){
			$this->ID = $groupID;
			$this->load($groupID);
		}
	}

	/*	 * **************************************
	 * load mailing list from database
	 *
	 * *************************************** */

	function load($groupID = 0){
		parent::load($groupID);
		$this->aFilter = we_unserialize($this->Filter);
		return true;
	}

	/*	 * **************************************
	 * save mailing list to database
	 *
	 * *************************************** */

	public function preserveFilter(){
		//make some fields up to date
		if(is_array($this->aFilter)){
			foreach($this->aFilter as &$v){
				switch(isset($v['fieldname']) ? $v['fieldname'] : ''){
					case 'MemberSince':
					case 'LastAccess':
					case 'LastLogin':
						if(!empty($v['fieldvalue'])){
							if(stristr($v['fieldvalue'], '.')){
								$date = explode('.', $v['fieldvalue']);
								$v['fieldvalue'] = mktime($v['hours'], $v['minutes'], 0, $date[1], $date[0], $date[2]);
							}
						}
				}
			}

			$this->Filter = we_serialize($this->aFilter, SERIALIZE_JSON);
		}
	}

	function save(){
		$this->preserveFilter();
		parent::save();
		return true;
	}

	function getFilter(){
		if($this->aFilter){
			return $this->aFilter;
		}
		$this->aFilter = we_unserialize($this->Filter);
		return $this->aFilter;
	}

	/*	 * **************************************
	 * delete mailing list from database
	 *
	 * *************************************** */

	function delete(){
		parent::delete();
		return true;
	}

	/*	 * **********************************
	 * check email syntax
	 *
	 * ********************************** */

	function checkEmails($group, &$malformed){
		$mails = [];
		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			if(empty($this->settings['customer_email_field'])){
				t_e('empty setting for customer email field');
			} else {
				$customers = array_map('intval', array_filter(explode(',', $this->Customers)));
				$mails = $this->db->getAllq('SELECT ' . $this->settings['customer_email_field'] . ' FROM ' . CUSTOMER_TABLE . ' WHERE ID IN(' . implode(',', $customers) . ')', true);
			}
		}

		$emails = array_merge($mails, we_newsletter_base::getEmailsFromList($this->Emails, 1), we_newsletter_newsletter::getEmailsFromExtern($this->Extern, 1));

		foreach($emails as $email){
			if(!$this->check_email($email)){
				$malformed = $email;
				return $group;
			}
		}
		return 0;
	}

	function addFilter($name = '', $pos = 0){
		$arr = $pos > 0 ? array_slice($this->aFilter, 0, $pos) : [];
		$arr[] = ['fieldname' => $name, 'operator' => 0, 'fieldvalue' => '', 'logic' => '', 'hours' => '', 'minutes' => ''];
		$this->aFilter = array_merge($arr, array_slice($this->aFilter, $pos));
	}

	function appendFilter(array $filter){
		$this->aFilter[] = $filter;
	}

	function delFilter($pos){
		unset($this->aFilter[$pos]);
		$this->aFilter = array_values($this->aFilter);
	}

	function delallFilter(){
		$this->Filter = '';
		$this->aFilter = [];
	}

//---------------------------------- STATIC FUNCTIONS -------------------------------

	/*	 * ****************************************************
	 * return all newsletter blocks for given newsletter id
	 *
	 * ***************************************************** */
	static function __getAllGroups($newsletterID, we_database_base $db){
		$db->query('SELECT ID FROM ' . NEWSLETTER_GROUP_TABLE . ' WHERE NewsletterID=' . intval($newsletterID) . ' ORDER BY ID');
		$ret = [];
		while($db->next_record()){
			$ret[] = new self($db->f('ID'));
		}
		return $ret;
	}

	static function getSettings(){
		$db = new DB_WE();
		$db->query('SELECT pref_name,pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="newsletter"');
		return $db->getAllFirst(false);
	}

}
