<?php

/**
 * webEdition CMS
 *
 * $Rev: 15093 $
 * $Author: mokraemer $
 * $Date: 2018-03-21 12:07:05 +0100 (Wed, 21 Mar 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tree_base{
	const DefaultWidth = 300;
	const MinWidth = 200;
	const MaxWidth = 1000;
	const StepWidth = 20;
	const DeleteWidth = 420;
	const MoveWidth = 500;
	const AddToCollectionWidth = 508;
	const HiddenWidth = 40;
	const MinWidthModules = 120;
	const MaxWidthModules = 800;
	const TYP_FOLDER = 'group';
	const TYP_ITEM = 'item';
	const TYP_DOTS = 'threedots';
	const CT_UP = 'arrowup';
	const CT_DOWN = 'arrowdown';

	protected $db;
	var $styles = [];
	var $tree_states = [
		'edit' => 0,
		'select' => 1,
		'selectitem' => 2,
		'selectgroup' => 3,
	];
	var $default_segment = 30;
	protected $autoload = true;
	protected $addSorted = true;
	protected $extraClasses = '';
	protected $jsCmd;

//Initialization

	public function __construct(we_base_jsCmd $jsCmd){
		$this->db = new DB_WE();
		$this->jsCmd = $jsCmd;
		$this->default_segment = intval(we_base_preferences::getUserPref('default_tree_count'));
	}

	/*
	  the functions prints tree javascript
	  should be placed in a frame which doesn't reloads

	 */

	public function getJSTreeCode(){
		if($this->autoload){
			$this->jsCmd->addCmd('loadTree', [
				'clear' => 1,
				'items' => static::getItems(0, 0, $this->default_segment),
				'sorted' => $this->addSorted
			]);
		}

		return we_html_element::jsScript(JS_DIR . 'tree.js') . $this->customJSFile();
	}

	abstract protected function customJSFile();

	public function getHTMLConstruct(){
		return
			we_html_element::cssLink(CSS_DIR . 'tree.css') .
			we_html_element::div(['id' => 'treetable',
				'class' => 'tree' . ($this->extraClasses ? ' ' . $this->extraClasses : ''),
				], ''
		);
	}

	/**
	 * @param $ParentID
	 * @param int $offset
	 * @param int $segment
	 * @param bool $sort
	 * @return array
	 */
	public static function getItems($ParentID, $offset = 0, $segment = 500, $sort = false){
		//overite
		return [];
	}

	public static function getParents($path, $forQuery = true){
		$out = [];
		$db = $GLOBALS['DB_WE'];
		while($path != '/' && $path != '\\' && $path){
			$out[] = ($forQuery ? '"' . $db->escape($path) . '"' : $path);
			$path = dirname($path);
		}
		return ($forQuery ? ($out ? 'Path IN(' . implode(',', $out) . ')' : '') : $out);
	}

}
