<?php

/**
 * webEdition CMS
 *
 * $Rev: 15829 $
 * $Author: mokraemer $
 * $Date: 2018-09-27 15:32:03 +0200 (Thu, 27 Sep 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_users_util{

	private static function getGroupList($id){
		if(!$id){
			return [];
		}
		$ret = [];
		$db_tmp = new DB_WE();
		$db_tmp->query('SELECT ID,username WHERE ParentID=' . intval($id) . ' AND Type=1');
		while($db_tmp->next_record()){
			$ret[$db_tmp->f('ID')] = $db_tmp->f('username');
			$section = self::getGroupList($db_tmp->f('ID'));
			$ret = array_merge($ret, $section);
		}
		return $ret;
	}

	private static function getUserTree($id){
		$ret = [];
		$db_tmp = new DB_WE();
		$db_tmp->query('SELECT ID,username,Type WHERE ParentID=' . intval($id));
		while($db_tmp->next_record()){
			$ret[$db_tmp->f('ID')] = ['name' => $db_tmp->f('username'),
				"ParentID" => $id,
				'type' => $db_tmp->f('Type')
			];
			$section = self::getUserTree($db_tmp->f('ID'));
			$ret = array_merge($ret, $section);
		}
		return $ret;
	}

	static function isUserInUsers($uid, $users, we_database_base $db = null){ // $users can be a csv string or an array
		if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
			return true;
		}
		$users = is_array($users) ? $users : array_filter(explode(',', $users));

		if(in_array($uid, $users)){
			return true;
		}
		$db = $db ?: new DB_WE();

		$aliases = self::getAliases($uid);
		foreach($aliases as $aid){
			if(in_array($aid, $users)){
				return true;
			}
		}

		foreach($users as $user){
			$isGroup = f('SELECT IsFolder FROM ' . USER_TABLE . ' WHERE ID=' . intval($user), "", $db);
			if($isGroup){
				if(self::isUserInGroup($uid, $user, $db)){
					return true;
				}
				foreach($aliases as $aid){
					if(self::isUserInGroup($aid, $user, $db)){
						return true;
					}
				}
			}
		}

		return false;
	}

	static function isUserInGroup($uid, $groupID, we_database_base $db = null){
		$db = $db ?: new DB_WE();
		$pid = f('SELECT ParentID FROM ' . USER_TABLE . ' WHERE ID=' . intval($uid), "ParentID", $db);
		if($pid == $groupID){
			return true;
		} else if($pid != 0){
			return self::isUserInGroup($pid, $groupID);
		}
		return false;
	}

	static function addAllUsersAndGroups($uid, &$arr){
		$db = new DB_WE();
		$db->query('SELECT ID,IsFolder FROM ' . USER_TABLE . ' WHERE ParentID=' . intval($uid));
		while($db->next_record()){
			$arr[] = $db->f('ID');
			if($db->f('IsFolder')){
				self::addAllUsersAndGroups($db->f('ID'), $arr);
			}
		}
	}

	static function removeNonAsociative(&$array){
		if(!is_array($array)){
			return $array;
		}

		foreach(array_keys($array) as $k){
			if((string) (int) $k == $k){
				unset($array[$k]);
			}
		}

		return $array;
	}

	static function getUsersForDocWorkspace(we_database_base $db, $id, $wsField = 'workSpace'){
		$ids = (is_array($id) ? $id : [$id]);

		$where = [];
		foreach($ids as $id){
			$where[] = 'FIND_IN_SET(' . intval($id) . ',' . $wsField . ')';
		}

		return $db->getAllFirstq('SELECT ID,username FROM ' . USER_TABLE . ' WHERE ' . ($where ? implode(' OR ', $where) : ''), false);
	}

	static function getAliases($id){
		return $GLOBALS['DB_WE']->getAllq('SELECT ID FROM ' . USER_TABLE . ' WHERE Alias=' . intval($id), true);
	}

	public static function isOwner($csvOwners){
		if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
			return true;
		}
		$ownersArray = array_filter(explode(',', $csvOwners));
		return (in_array($_SESSION['user']['ID'], $ownersArray)) || self::isUserInUsers($_SESSION['user']['ID'], $csvOwners);
	}

	public static function userIsOwnerCreatorOfParentDir($folderID, $tab){
		if(($tab !== FILE_TABLE && $tab !== OBJECT_FILES_TABLE) ||
			(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) || ($folderID == 0))){
			return true;
		}
		$db = new DB_WE();
		$tmp = getHash('SELECT RestrictOwners,Owners,CreatorID FROM ' . $tab . ' WHERE ID=' . intval($folderID), $db);
		if(!$tmp){
			return true;
		}
		if($tmp['RestrictOwners']){
			$ownersArr = explode(',', $tmp['Owners']);
			foreach($ownersArr as $uid){
				we_users_util::addAllUsersAndGroups($uid, $ownersArr);
			}
			$ownersArr[] = $tmp['CreatorID'];
			$ownersArr = array_unique($ownersArr);
			return (in_array($_SESSION['user']['ID'], $ownersArr));
		}
		$pid = f('SELECT ParentID FROM ' . $tab . ' WHERE ID=' . intval($folderID), '', $db);
		return self::userIsOwnerCreatorOfParentDir($pid, $tab);
	}

	public static function canEditModule($modName){
		if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
			return true;
		}
		$perm = we_modules_info::getModuleField($modName, 'perm');
		return we_gui_menu::isEnabled(is_array($perm) ? $perm : [$perm]);
	}

	public static function makeOwnersSql($useCreatorID = true){
		if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
			return '';
		}
		$aliases = self::getAliases($_SESSION['user']['ID']);
		$aliases[] = $_SESSION['user']['ID'];
		$q = [];
		if($useCreatorID){
			$q[] = 'CreatorID IN ("' . implode('","', $aliases) . '")';
		}
		foreach($aliases as $id){
			$q[] = 'FIND_IN_SET(' . intval($id) . ',Owners)';
		}
		$groups = [$_SESSION['user']['ID']];
		we_getParentIDs(USER_TABLE, $_SESSION['user']['ID'], $groups, $GLOBALS['DB_WE']);
		foreach($aliases as $id){
			we_getParentIDs(USER_TABLE, $id, $groups, $GLOBALS['DB_WE']);
		}

		foreach($groups as $id){
			$q[] = 'FIND_IN_SET(' . intval($id) . ',Owners)';
		}
		return ' AND ( RestrictOwners=0 OR (RestrictOwners=1 AND (' . implode(' OR ', $q) . '))) ';
	}

	public static function getAllowedClasses(we_database_base $db){
		if(!we_modules_info::isActive(we_modules_info::OBJECT)){
			return [];
		}
		$ofWs = get_ws(OBJECT_FILES_TABLE);
		//return all classes if admin, or no restrictions
		if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) || !$ofWs){
			return $db->getAllq('SELECT ID FROM ' . OBJECT_TABLE, true);
		}
		$ret = $db->getAllq('SELECT DISTINCT TableID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID IN (' . $ofWs . ')', true);
		//if workspace don't exist, make sure we don't show classes
		return $ret ?: ['-1'];
	}

	public static function in_workspace($ID, array $wsIDs, $table = FILE_TABLE, $norootcheck = false){
		if(empty($wsIDs) || (in_array(0, $wsIDs))){
			return true;
		}
		$ID = intval($ID);

		if($ID === 0){
			return (!$norootcheck ? false : true);
		}

		foreach($wsIDs as $ws){
			if(($ID == $ws) || in_parentID($ID, $ws, $table)){
				return true;
			}
		}
		return false;
	}

	public static function changeRecursive(){
		$ok = false;

		if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
			$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', 0, 1);

			// init document
			$we_dt = $_SESSION['weS']['we_data'][$we_transaction];
			$we_doc = we_contents_document::initDoc($we_dt);

			$childs = [];
			pushChilds($childs, $we_doc->ID, $we_doc->Table, false, $GLOBALS['DB_WE']);
			if($childs){
				$ok = $GLOBALS['DB_WE']->query('UPDATE ' . $we_doc->Table . ' SET ' .
					we_database_base::arraySetter(['CreatorID' => $we_doc->CreatorID,
						'Owners' => $we_doc->Owners,
						'RestrictOwners' => $we_doc->RestrictOwners,
						'OwnersReadOnly' => $we_doc->OwnersReadOnly
					]) . ' WHERE ID IN(' . implode(',', $childs) . ')');
			}
		}
		$jsCmd = new we_base_jsCmd();
		$jsCmd->addMsg(g_l('modules_users', '[grant_owners_ok]'), ($ok ? we_base_util::WE_MESSAGE_NOTICE : we_base_util::WE_MESSAGE_ERROR));

		return we_html_tools::getHtmlTop('', '', '', $jsCmd->getCmds(), we_html_element::body());
	}

}
