<?php

/**
 * webEdition CMS
 *
 * $Rev: 15647 $
 * $Author: mokraemer $
 * $Date: 2018-08-27 12:09:28 +0200 (Mon, 27 Aug 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_ifVar extends we_weTag_ifTag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(($foo = self::attributeMissing($attribs, ['name' => false, 'match' => true], __CLASS__))){
			echo $foo;
			return false;
		}

		$match_orig = self::getAttribute('match', $attribs, '', we_base_request::STRING);
		$type = self::getAttribute('type', $attribs, 'document', we_base_request::STRING);
		$operator = self::getAttribute('operator', $attribs, 'equal', we_base_request::STRING);

		if(is_bool($match_orig)){
			$size = 1;
		} elseif(is_array($match_orig)){
			$matchArray = $match_orig;
			$size = count($matchArray);
		} else {
			$matchArray = makeArrayFromCSV($match_orig);
			$size = count($matchArray);
			$regs = [];
			if($size == 1 && preg_match('|^\\\\?\$([^\[]+)(\[.*\])?|', $match_orig, $regs)){
				$match_orig = (isset($regs[2]) ?
					self::getArrayValue($GLOBALS, $regs[1], $regs[2]) :
					(isset($GLOBALS[$regs[1]]) ? $GLOBALS[$regs[1]] : ''));
			}
		}

		$nameOrig = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);

		switch($type){
			case 'customer' :
			case 'sessionfield' :
				return ($size == 1 && isset($_SESSION['webuser'][$nameOrig]) ?
					self::op($operator, $_SESSION['webuser'][$nameOrig], $match_orig) :
					(isset($_SESSION['webuser'][$nameOrig]) && in_array($_SESSION['webuser'][$nameOrig], $matchArray)));

			case 'global':
				$var = self::getArrayValue($GLOBALS, null, $nameOrig);
				return ($size == 1 ?
					self::op($operator, $var, $match_orig) :
					(!empty($var) && in_array($var, $matchArray)));

			case 'request':
				$var = self::getArrayValue($_REQUEST, null, $nameOrig);
				return ($size == 1 ?
					self::op($operator, $var, $match_orig) :
					(!empty($var) && in_array($var, $matchArray)));

			case 'post':
				$var = self::getArrayValue($_POST, null, $nameOrig);
				return ($size == 1 ?
					self::op($operator, $var, $match_orig) :
					(!empty($var) && in_array($var, $matchArray)));

			case 'get':
				$var = self::getArrayValue($_GET, null, $nameOrig);
				return ($size == 1 ?
					self::op($operator, $var, $match_orig) :
					(!empty($var) && in_array($var, $matchArray)));

			case 'session':
				$var = self::getArrayValue($_SESSION, null, $nameOrig);
				return ($size == 1 ?
					self::op($operator, $var, $match_orig) :
					(!empty($var) && in_array($var, $matchArray)));

			case 'property':
				$doc = self::getDocForTag(self::getAttribute('doc', $attribs, '', we_base_request::STRING), true);
				if(!isset($doc->$nameOrig)){
					return false;
				}
				$var = $doc->$nameOrig;
				return ($size == 1 ?
					self::op($operator, $var, $match_orig) :
					in_array($var, $matchArray));

			case 'document':
			default:
				$doc = self::getDocForTag(self::getAttribute('doc', $attribs, '', we_base_request::STRING));
				$val = $doc->getField($attribs, $type, true);

				return ($size == 1 ?
					self::op($operator, $val, $match_orig) :
					in_array($val, $matchArray));
		}
	}

	private static function op($operator, $first, $match){
		switch($operator){
			default:
			case 'equal':
				return $first == $match;
			case 'less':
				return $first < $match;
			case 'less|equal':
				return $first <= $match;
			case 'greater':
				return $first > $match;
			case 'greater|equal':
				return $first >= $match;
			case 'contains':
				return (strpos($first, $match) !== false);
			case 'isin':
				return (strpos($match, $first) !== false);
		}
	}

}

function we_tag_ifVar(array $attribs){
	return we_tag_ifVar::tag($attribs);
}
