<?php

/**
 * webEdition CMS
 *
 * $Rev: 15713 $
 * $Author: mokraemer $
 * $Date: 2018-09-05 14:54:44 +0200 (Wed, 05 Sep 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_object extends we_weTag_tag{

	public static function parse(array $attribs, $content){
		$name = self::getParserAttribute('name', $attribs);
		if($name && strpos($name, ' ') !== false){
			return self::parseError(sprintf(g_l('parser', '[name_with_space]'), 'object'), __CLASS__);
		}

		//dont't check if id or name is set, since it is possible to set object ID by request

		return '<?php ' . (strpos($content, '$lv') !== false ? 'global $lv;' : '') .
			'if(' . we_weTag_parser::printTag('object', $attribs) . '){?>' . $content . '<?php }
		we_weTag_tag::post_listview(); ?>';
	}

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(!we_modules_info::isActive(we_modules_info::OBJECT)){
			echo self::modulMissingError('Object/DB', __CLASS__);
			return false;
		}

		$condition = self::getAttribute('condition', $attribs, 0, we_base_request::RAW);
		$classid = self::getAttribute('classid', $attribs, 0, we_base_request::INT);
		$we_oid = self::getAttribute('id', $attribs, 0, we_base_request::INT);
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$size = 5 * intval(self::getAttribute('size', $attribs, 30, we_base_request::UNIT));
		$triggerid = self::getAttribute('triggerid', $attribs, 0, we_base_request::INT);
		$searchable = self::getAttribute('searchable', $attribs, false, we_base_request::BOOL);
		$hidedirindex = self::getAttribute('hidedirindex', $attribs, TAGLINKS_DIRECTORYINDEX_HIDE, we_base_request::BOOL);
		$objectseourls = self::getAttribute('objectseourls', $attribs, TAGLINKS_OBJECTSEOURLS, we_base_request::BOOL);

		if($name){
			if(strpos($name, ' ') !== false){
				echo self::parseError(sprintf(g_l('parser', '[name_with_space]'), 'object'), __CLASS__);
				return false;
			}

			$we_doc = $GLOBALS['we_doc'];
			//handle listview of documents
			$we_oid = intval(we_tag_listview::$current && we_tag_listview::$current instanceof we_listview_base && we_tag_listview::$current->f($name) ?
				we_tag_listview::$current->f($name) :
				($we_doc->getElement($name, we_contents_base::ETYPE_NUMBER) ?:
				($we_doc->getElement($name) ?:
				$we_oid)
				));
			$rootDirID = ($classid ? f('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE IsClassFolder=1 AND TableID=' . intval($classid)) : 0);

			$path = f('SELECT Path FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $we_oid);
			$textname = 'we_' . $we_doc->Name . '_' . we_contents_base::INTERNAL_VAR . '[' . $name . '_path]';
			$idname = 'we_' . $we_doc->Name . '_object[' . $name . '#' . we_contents_base::ETYPE_NUMBER . ']';

			if(!empty($GLOBALS['we']['editmode'])){
				$weSuggest = &we_gui_suggest::getInstance();
				$weSuggest->setAcId($name . we_base_file::getUniqueId(), f('SELECT Path FROM ' . OBJECT_TABLE . ' WHERE ID=' . $classid));
				$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::OBJECT_FILE]);
				$weSuggest->setInput($textname, $path);
				$weSuggest->setResult($idname, $we_oid);
				$weSuggest->setMaxResults(10);
				$weSuggest->setSelector(we_gui_suggest::DocSelector);
				$weSuggest->setTable(OBJECT_FILES_TABLE);
				$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $idname . "'].value,WE().consts.tables.OBJECT_FILES_TABLE,'" . $idname . "','" . $textname . "','reload_hot_editpage','','" . $rootDirID . "',WE().consts.contentTypes.OBJECT_FILE," . (we_base_permission::hasPerm(we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS) ? 0 : 1) . ")"));
				$weSuggest->setOpenButton(we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:if(document.forms[0].elements['" . $idname . "'].value){WE().layout.weEditorFrameController.openDocument(WE().consts.tables.OBJECT_FILES_TABLE, document.forms[0].elements['" . $idname . "'].value,'');}"));
				$weSuggest->setCreateButton(we_html_button::create_button(we_html_button::ADD, "javascript:top.we_cmd('new',WE().consts.tables.OBJECT_FILES_TABLE,'',WE().consts.contentTypes.OBJECT_FILE," . $classid . ");"));
				$weSuggest->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.forms[0].elements['" . $idname . "'].value=0;document.forms[0].elements['" . $textname . "'].value='';_EditorFrame.setEditorIsHot(false);we_cmd('reload_editpage');"));
				$weSuggest->setWidth($size);
//				ob_start();
				?>
				<table class="we we_object weEditTable">
					<tr>
						<td><span class="bold"><?= self::getAttribute('text', $attribs, self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING), we_base_request::STRING); ?></span></td>
						<td><?= $weSuggest->getHTML(); ?></td>
					</tr>
				</table><?php
				//return ob_get_clean();
				//return
			}
		}

		$we_oid = $we_oid ?:
//Fix #10526 check if objectID is given by request
			(we_base_request::_(we_base_request::INT, WEOBJ_PARAM) ?: we_base_request::_(we_base_request::INT, 'we_oid', 0));

		//Fix #10609 we need classID now!
		$classid = $classid ?: ($we_oid ? f('SELECT TableID FROM ' . OBJECT_FILES_TABLE . ' WHERE IsFolder=0 AND ID=' . intval($we_oid)) : 0);

		if($we_oid && $classid){
			$unique = md5(uniqid(__CLASS__, true));
			$lv = new we_listview_object($unique, 1, 0, '', 0, $classid, '', '', 'ID=' . intval($we_oid) . ($condition ? ' AND ' . $condition : ''), $triggerid, '', '', $searchable, '', '', '', '', '', [
				], '', 0, '', '', '', '', $hidedirindex, $objectseourls, true);
			$avail = $lv->next_record();

			if($avail){
				if(isset($_SESSION['weS']['we_mode']) && $_SESSION['weS']['we_mode'] === we_base_constants::MODE_SEE){
					echo we_gui_SEEM::getSeemAnchors($we_oid, 'object');
				}
			}
			self::pre_listview($lv);
			return $avail;
		}
		self::pre_listview(new stdClass());
		return false;
	}

}

function we_tag_object(array $attribs){
	return we_tag_object::tag($attribs);
}
