<?php

/**
 * webEdition CMS
 *
 * $Rev: 15663 $
 * $Author: mokraemer $
 * $Date: 2018-08-28 20:01:53 +0200 (Tue, 28 Aug 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_setVar extends we_weTag_tag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(($foo = self::attributeMissing($attribs, ['nameto' => false, 'to' => false], __CLASS__))){
			return $foo;
		}

		$nameFrom = self::getAttribute('namefrom', $attribs, '', we_base_request::STRING);
		$nameTo = self::getAttribute('nameto', $attribs, '', we_base_request::STRING);
		$typeFrom = self::getAttribute('typefrom', $attribs, 'text', we_base_request::STRING);
		$to = self::getAttribute('to', $attribs, '', we_base_request::STRING);
		$from = self::getAttribute('from', $attribs, '', we_base_request::STRING);
		$propertyTo = self::getAttribute('propertyto', $attribs, false, we_base_request::BOOL);
		$propertyFrom = self::getAttribute('propertyfrom', $attribs, false, we_base_request::BOOL);
		$striptags = self::getAttribute('striptags', $attribs, false, we_base_request::BOOL);
		$formnameTo = self::getAttribute('formnameto', $attribs, we_base_constants::DEFAULT_FORM, we_base_request::STRING);
		$formnameFrom = self::getAttribute('formnamefrom', $attribs, we_base_constants::DEFAULT_FORM, we_base_request::STRING);
		$varType = $typeFrom === 'href' ? we_base_request::URL : self::getAttribute('varType', $attribs, we_base_request::STRING, we_base_request::STRING);
		$prepareSQL = self::getAttribute('prepareSQL', $attribs, false, we_base_request::BOOL);

		if(isset($attribs['value'])){
			$valueFrom = self::getAttribute('value', $attribs, '', we_base_request::RAW);
		} else {
			switch($from){
				case 'request':
				case 'post':
				case 'get':
				case 'global':
				case 'session':
					$valueFrom = we_tag('var', ['type' => $from, '_name_orig' => $nameFrom, 'name' => $nameFrom, 'varType' => $varType]);
					break;
				case 'top':
					if($propertyFrom){
						$valueFrom = isset($GLOBALS['we']['main']->$nameFrom) ? $GLOBALS['we']['main']['doc']->$nameFrom : '';
					} else {
						$valueFrom = $GLOBALS['we']['main']['doc']->issetElement($nameFrom . ($typeFrom === 'href' ? we_base_link::MAGIC_INT_LINK : '')) ?
							$GLOBALS['we']['main']['doc']->getField(['name' => $nameFrom], $typeFrom, true) :
							'';
					}
					break;
				case 'self':
					if($propertyFrom){
						$valueFrom = isset($GLOBALS['we_doc']->$nameFrom) ? $GLOBALS['we_doc']->$nameFrom : '';
					} else {
						$valueFrom = $GLOBALS['we_doc']->issetElement($nameFrom . ($typeFrom === 'href' ? we_base_link::MAGIC_INT_LINK : '')) ?
							$GLOBALS['we_doc']->getField(['name' => $nameFrom], $typeFrom, true) :
							'';
					}
					break;
				case 'object':
				case 'document':
					$valueFrom = ($propertyFrom ?
						(isset($GLOBALS['we_' . $from][$formnameFrom]->$nameFrom) ? $GLOBALS['we_' . $from][$formnameFrom]->$nameFrom : '') :
						(is_object($GLOBALS['we_' . $from][$formnameFrom]) && $GLOBALS['we_' . $from][$formnameFrom]->issetElement($nameFrom) ? $GLOBALS['we_' . $from][$formnameFrom]->getElement($nameFrom) : ''));

					break;
				case 'sessionfield':
					$valueFrom = isset($_SESSION['webuser'][$nameFrom]) ? $_SESSION['webuser'][$nameFrom] : '';
					break;
				case 'calendar':
					list(, $langcode) = self::getOutLang($attribs);
					$valueFrom = we_tag_listview::$current->getCalendarFieldValue($nameFrom, $langcode);
					break;
				case 'listview':
					if(!we_tag_listview::$current){
						return self::parseError(g_l('parser', '[setVar_lv_not_in_lv]'), __CLASS__);
					}
					$valueFrom = we_tag('field', ['name' => $nameFrom, 'type' => $typeFrom]);
					break;
				case 'block':
					$nameFrom = self::getPostName($nameFrom);
					if($typeFrom === 'href' && $GLOBALS['we_doc']->getElement($nameFrom . we_base_link::MAGIC_INT_LINK, we_contents_base::ETYPE_NUMBER)){
						$nameFrom .= we_base_link::MAGIC_INT_LINK_ID;
					}
					$valueFrom = $GLOBALS['we']['main']['doc']->issetElement($nameFrom) ? $GLOBALS['we']['main']['doc']->getField(['name' => $nameFrom], $typeFrom, true) : '';
					break;
				case 'listdir':
					$valueFrom = isset(we_tag_listdir::$position[$nameFrom]) ? we_tag_listdir::$position[$nameFrom] : '';
					break;
				default:
					$valueFrom = '';
			}
		}

		$valueFrom = ($striptags ? strip_tags($valueFrom) : $valueFrom);
		$valueFrom = ($prepareSQL ? $GLOBALS['DB_WE']->escape($valueFrom) : $valueFrom);

		switch($to){
			case 'object':
			case 'document':
				if(isset($GLOBALS['we_' . $to][$formnameTo])){
					if($propertyTo){
						$GLOBALS['we_' . $to][$formnameTo]->$nameTo = $valueFrom;
					} else {
						$GLOBALS['we_' . $to][$formnameTo]->setElement($nameTo, $valueFrom);
					}
				}
				break;
			case 'top' :
			case 'self' :
				if($propertyTo){
					if($to === 'top'){
						$GLOBALS['we']['main']['doc']->$nameTo = $valueFrom;
					} else {
						$GLOBALS['we_doc']->$nameTo = $valueFrom;
					}
					break;
				}
			default:
				self::redirectTagOutput($valueFrom, $nameTo, $to);
		}
		return '';
	}

}

function we_tag_setVar(array $attribs){
	return we_tag_setVar::tag($attribs);
}
