<?php

/**
 * webEdition CMS
 *
 * $Rev: 15714 $
 * $Author: mokraemer $
 * $Date: 2018-09-05 15:22:25 +0200 (Wed, 05 Sep 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_shopField extends we_weTag_tag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(($foo = self::attributeMissing($attribs, ['name' => false, 'reference' => false, 'shopname' => false], __CLASS__))){
			return $foo;
		}

		$name = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
		$reference = self::getAttribute('reference', $attribs, '', we_base_request::STRING);
		$shopname = self::getAttribute('shopname', $attribs, '', we_base_request::STRING);

		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);

		if($type === 'checkbox' && ($missingAttrib = self::attributeMissing($attribs, 'value', __CLASS__))){
			echo $missingAttrib;
		}

		$values = self::getAttribute('values', $attribs, [], we_base_request::STRING_LIST); // select, choice
		$value = oldHtmlspecialchars(self::getAttribute('value', $attribs, '', we_base_request::RAW), -1, 'ISO-8859-1', false); // checkbox
		$checked = self::getAttribute('checked', $attribs, false, we_base_request::BOOL); // checkbox

		if($checked && ($foo = self::attributeMissing($attribs, 'value', __CLASS__))){
			return $foo;
		}
		$mode = self::getAttribute('mode', $attribs, '', we_base_request::STRING);

		$fieldname = ($reference === 'article' ? we_shop_shop::ARTICLE_CUSTOM_FIELD : we_shop_shop::CART_CUSTOM_FIELD) . '[' . $name . ']';

		$isFieldForCheckBox = false;

		if($reference === 'article'){ // name depends on value
			$orgVal = (!$shopname) ? we_base_request::_(we_base_request::HTML, we_shop_shop::ARTICLE_CUSTOM_FIELD, '', $name) : '';
			// does not exist here - we are only in article - custom fields are not stored on documents

			if(we_tag_listview::$current && ($tmpVal = we_tag('field', ['name' => $name]))){
				$orgVal = $tmpVal;
				unset($tmpVal);
			}
		} else {
			$orgVal = isset($GLOBALS[$shopname]) ? $GLOBALS[$shopname]->getCartField($name) : '';
			$isFieldForCheckBox = isset($GLOBALS[$shopname]) ? $GLOBALS[$shopname]->hasCartField($name) : false;
		}

		$atts = removeAttribs($attribs, ['name', 'reference', 'shopname', 'type', 'values', 'value', 'checked', 'mode']);

		if($type != 'checkbox' && $type != 'choice' && $type != 'radio' && $value){
			// value is compared to saved value in some cases
			// be careful with different behaviour when using value and values
			if(!$orgVal){
				$orgVal = $value;
			}
		}

		switch($type){
			case 'checkbox':
				$atts = removeAttribs($atts, ['size']);
				//$atts['name'] = $fieldname; changed to $tnpname because of new hidden field #6544
				//we_getInputCheckboxField() not possible because sessionField type="checkbox" has a mandatory value
				$tmpname = md5(uniqid(__CLASS__, true)); // #6590, changed from: uniqid(time())
				$atts['name'] = $tmpname;
				$atts['type'] = 'checkbox';
				$atts['value'] = $value;
				$atts['onclick'] = 'this.form.elements[\'' . $fieldname . '\'].value=(this.checked) ? \'' . oldHtmlspecialchars($value) . '\' : \'\''; //#6544
				if(($orgVal == $value) || (!$isFieldForCheckBox) && $checked){
					$atts['checked'] = 'checked';
				}

				return we_html_element::span(['class' => 'we we_shopField'], getHtmlTag('input', $atts) . we_html_element::hidden($fieldname, $orgVal));

			case 'choice':
				return we_html_element::span(['class' => 'we we_shopField'], we_html_tools::inputChoiceField($fieldname, $orgVal, $values, $atts, $mode));

			case 'hidden':
				$atts = removeAttribs($atts, ['reference']);
				$atts['name'] = $fieldname;
				$atts['value'] = $orgVal;
				$atts['type'] = 'hidden';
				return getHtmlTag('input', $atts);

			case 'print':
				$ascountry = self::getAttribute('ascountry', $attribs, false, we_base_request::BOOL);
				$aslanguage = self::getAttribute('aslanguage', $attribs, false, we_base_request::BOOL);
				if($ascountry || $aslanguage){
					$lang = self::getAttribute('outputlanguage', $attribs, '', we_base_request::STRING);
					if(!$lang){
						$doc = self::getDocForTag(self::getAttribute('doc', $attribs, 'self', we_base_request::STRING));
						$lang = $doc->Language;
					}
					$langcode = substr($lang, 0, 2);
					if(!$lang){
						$lang = explode('_', $GLOBALS['WE_LANGUAGE']);
						$langcode = array_search($lang[0], getWELangs());
					}
					return ($ascountry && $orgVal === '--' ? '' : CheckAndConvertISOfrontend(we_base_country::getTranslation($orgVal, ($ascountry ? we_base_country::TERRITORY : we_base_country::LANGUAGE), $langcode)));
				}
				return $orgVal;

			case 'select':
				return we_html_element::span(['class' => 'we we_shopField'], self::selectField($fieldname, $orgVal, $values, $atts, false));

			case 'country':
				$newAtts = removeAttribs($attribs, ['name', 'type', 'value', 'values', 'checked', 'mode']);
				$newAtts['name'] = we_shop_shop::CART_CUSTOM_FIELD . '[' . $name . ']';
				$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
				$doc = self::getDocForTag($docAttr);
				$lang = $doc->Language;
				$langcode = ($lang ?
					substr($lang, 0, 2) :
					array_search($GLOBALS['WE_LANGUAGE'], getWELangs()));

				$topCountries = array_flip(explode(',', WE_COUNTRIES_TOP));
				foreach($topCountries as $countrykey => &$countryvalue){
					$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
				}
				unset($countryvalue);

				$shownCountries = array_flip(explode(',', WE_COUNTRIES_SHOWN));
				foreach($shownCountries as $countrykey => &$countryvalue){
					$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
				}
				unset($countryvalue);
				$oldLocale = setlocale(LC_ALL, NULL);
				setlocale(LC_ALL, $lang . '.UTF-8');
				asort($topCountries, SORT_LOCALE_STRING);
				asort($shownCountries, SORT_LOCALE_STRING);
				setlocale(LC_ALL, $oldLocale);

				$content = '';
				if(WE_COUNTRIES_DEFAULT != ''){
					$content .= '<option value="--" ' . ($orgVal === '--' ? ' selected="selected">' : '>') . WE_COUNTRIES_DEFAULT . '</option>';
				}
				foreach($topCountries as $countrykey => &$countryvalue){
					$content .= '<option value="' . $countrykey . '" ' . ($orgVal == $countrykey ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($countryvalue) . '</option>';
				}
				unset($countryvalue);

				if(!empty($topCountries) && !empty($shownCountries)){
					$content .= '<option value="-" disabled="disabled">----</option>';
				}
				foreach($shownCountries as $countrykey2 => &$countryvalue2){
					$content .= '<option value="' . $countrykey2 . '" ' . ($orgVal == $countrykey2 ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($countryvalue2) . '</option>';
				}
				unset($countryvalue2);

				return we_html_element::span(['class' => 'we we_shopField'], getHtmlTag('select', $newAtts, $content, true));

			case 'language':
				$newAtts = removeAttribs($attribs, ['name', 'type', 'value', 'values', 'checked', 'mode']);
				$newAtts['name'] = we_shop_shop::CART_CUSTOM_FIELD . '[' . $name . ']';
				$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
				$doc = self::getDocForTag($docAttr);
				$lang = $doc->Language;
				$langcode = ($lang ?
					substr($lang, 0, 2) :
					array_search($GLOBALS['WE_LANGUAGE'], getWELangs()));

				$frontendL = $GLOBALS['we']['lang']['FrontendLanguages'];
				foreach($frontendL as &$lcvalue){
					$lccode = explode('_', $lcvalue);
					$lcvalue = $lccode[0];
				}
				$frontendLL = [];
				foreach($frontendL as &$lcvalue){
					$frontendLL[$lcvalue] = we_base_country::getTranslation($lcvalue, we_base_country::LANGUAGE, $langcode);
				}

				$oldLocale = setlocale(LC_ALL, NULL);
				setlocale(LC_ALL, $lang . '.UTF-8');
				asort($frontendLL, SORT_LOCALE_STRING);
				setlocale(LC_ALL, $oldLocale);
				$content = '';
				foreach($frontendLL as $langkey => &$langvalue){
					$content .= '<option value="' . $langkey . '" ' . ($orgVal == $langkey ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($langvalue) . '</option>' . "\n";
				}
				unset($langvalue);
				return we_html_element::span(['class' => 'we we_shopField'], getHtmlTag('select', $newAtts, $content, true));

			case 'textarea':
				return we_html_element::span(['class' => 'we we_shopField'], self::textareaField($fieldname, $orgVal, $atts));

			case 'radio':
				if($checked && $orgVal === ''){
					$atts['checked'] = 'checked';
				}
				return we_html_element::span(['class' => 'we we_shopField'], self::inputRadioField($fieldname, $orgVal, $value, $atts));

			case 'textinput':
			default:
				return we_html_element::span(['class' => 'we we_shopField'], self::inputTextInputField($fieldname, $orgVal, $atts));
		}
	}

}

function we_tag_shopField(array $attribs){
	return we_tag_shopField::tag($attribs);
}
