<?php

/**
 * webEdition CMS
 *
 * $Rev: 15830 $
 * $Author: mokraemer $
 * $Date: 2018-09-27 15:50:42 +0200 (Thu, 27 Sep 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_tag_xmlnode extends we_weTag_tag{
	private $struct = null;
	private $to = '';
	private $nameTo = '';
	private static $xpaths = [];
	private static $xstack = [];
	private static $xsuperparent = null;
	private static $instances = [];

	private function __construct(_we_tag_xmlnode_struct $struct, $to, $nameTo){
		$this->struct = $struct;
		$this->to = $to;
		$this->nameTo = $nameTo;
		//append this instance
		self::$instances[] = $this;
		ob_start();
	}

	private function doRun(){
		while($this->struct->next()){
			if($this->struct->hasChild()){
				self::$xsuperparent = $this->struct->getNode();
				return true;
			}
			echo $this->struct->getFeedData();
		}
		echo self::redirectTagOutput(ob_get_clean(), $this->nameTo, $this->to);
		//remove this instance
		array_pop(self::$instances);
		return false;
	}

	public static function run(){
		return end(self::$instances)->doRun();
	}

	public static function parse(array $attribs, $content){
		$unq = '$xmlnode' . md5(uniqid(__CLASS__, true));
		return '<?php ' . we_weTag_parser::printTag('xmlnode', $attribs) . ';
while(we_tag_xmlnode::run()){
?>' . $content . '<?php
}?>';
	}

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(($foo = self::attributeMissing($attribs, 'xpath', __CLASS__))){
			echo $foo;
			return false;
		}
		$feed = self::getPostName(self::getAttribute('feed', $attribs, '', we_base_request::STRING));
		$url = self::getAttribute('url', $attribs, '', we_base_request::URL);
		$encoding = (empty($GLOBALS['we_doc']) ? 'UTF-8' : $GLOBALS['we_doc']->getElement('Charset', we_contents_base::ETYPE_TEXT, DEFAULT_CHARSET, true));

		$pind_name = count(self::$xstack) - 1;
		if($pind_name < 0){
			$pind_name = 0;
			$parent_name = '';
		} else {
			$parent_name = self::$xstack[$pind_name];
		}

		$ind_name = count(self::$xpaths) + 1;
		self::$xpaths[$ind_name] = [
			'xpath' => $attribs['xpath'],
			'parent' => $parent_name
		];

		// find feed
		if($url){
			$feed_name = new we_xml_browser($url);
			self::$xpaths[$ind_name]['url'] = $url;
			$got_name = true;
		} elseif($feed){
			$feed_name = we_tag_xmlfeed::$xmlfeeds[$feed];
			self::$xpaths[$ind_name]['feed'] = $feed;
			$got_name = true;
		} else {
			$got_name = false;
			$c_name = 0;

			if(!empty($parent_name)){
				for($c_name = $pind_name; $c_name > -1; $c_name--){
					$otac_name = self::$xstack[$c_name];
					if(isset(self::$xpaths[$otac_name])){
						if(!empty(self::$xpaths[$otac_name]['url'])){
							$feed_name = new we_xml_browser(self::$xpaths[$otac_name]['url']);
							self::$xpaths[$ind_name]['url'] = self::$xpaths[$otac_name]['url'];
							$got_name = true;
						}
						if(!empty(self::$xpaths[$otac_name]['feed'])){
							$feed_name = we_tag_xmlfeed::$xmlfeeds[self::$xpaths[$otac_name]['feed']];
							self::$xpaths[$ind_name]['feed'] = self::$xpaths[$otac_name]['feed'];
							$got_name = true;
						}
					}
				}
			}
		}
		$nodes_name = [];
		if($got_name){
			if(self::$xsuperparent !== null){
				$nodes_name = $feed_name->evaluate(self::$xsuperparent . '/' . self::$xpaths[$ind_name]['xpath']);
			}
			if(empty($nodes_name)){
				$nodes_name = $feed_name->evaluate(self::$xpaths[$ind_name]['xpath']);
			}
			if(empty($nodes_name) && !empty($parent_name)){
				for($c_name = $pind_name; $c_name > -1; $c_name--){
					$otac_name = self::$xstack[$c_name];
					if(isset(self::$xpaths[$otac_name])){
						if(!empty(self::$xpaths[$otac_name]['xpath'])){
							self::$xpaths[$ind_name]['xpath'] = self::$xpaths[$otac_name]['xpath'] . '/' . self::$xpaths[$ind_name]['xpath'];
							$nodes_name = $feed_name->evaluate(self::$xpaths[$ind_name]['xpath']);
						}
					}
				}
			}
		}

		self::$xstack[] = $ind_name; //war einfach ind_name und fehler undefinend konstant

		new self(new _we_tag_xmlnode_struct($nodes_name, $feed_name, $encoding), self::getAttribute('to', $attribs, 'screen'), self::getAttribute('nameto', $attribs)
		);
	}

}

class _we_tag_xmlnode_struct{
	private $nodes_name;
	private $feed_name;
	private $init = false;
	private $xmlCharset = '';
	private $docCharset = '';

	function __construct($nodes_name, $feed_name, $encoding){
		$this->nodes_name = $nodes_name;
		$this->feed_name = $feed_name;
		$this->xmlCharset = $this->feed_name->mainXmlEncoding;
		$this->docCharset = $encoding;
	}

	function next(){
		if($this->init){
			return next($this->nodes_name) !== false;
		}
		$this->init = true;
		return reset($this->nodes_name) !== false;
	}

	function hasChild(){
		return $this->feed_name->hasChildNodes(current($this->nodes_name));
	}

	function getFeedData(){
		$ret = $this->feed_name->getData(current($this->nodes_name));
		return (
			$this->docCharset === $this->xmlCharset ?
			$ret :
			mb_convert_encoding($ret, $this->docCharset, $this->xmlCharset)
			);
	}

	function getNode(){
		return current($this->nodes_name);
	}

}

/**
 * @return string
 * @param array $attribs
 * @param string $content
 */
function we_tag_xmlnode(array $attribs){
	return we_tag_xmlnode::tag($attribs);
}
