<?php

/**
 * webEdition CMS
 *
 * $Rev: 15598 $
 * $Author: mokraemer $
 * $Date: 2018-08-21 18:22:59 +0200 (Tue, 21 Aug 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_workflow_view extends we_modules_view{
	const PAGE_PROPERTIES = 0;
	const PAGE_OVERVIEW = 1;

	// workflow array; format workflow[workflowID]=workflow_name
	var $workflows = [];
	//default document
	var $documentDef;
	//what is current display 0-workflow(default);1-document;
	var $show = 0;
	//wat page is currentlly displed 0-properties(default);1-overview;
	var $page = self::PAGE_PROPERTIES;
	var $hiddens = [];
	var $uid;

	function __construct(){
		parent::__construct();
		$this->uid = 'wf_' . md5(uniqid(__FILE__, true));
		//default workflow
		$this->Model = new we_workflow_workflow();
		$this->documentDef = new we_workflow_document();
		$this->hiddens = array_merge($this->hiddens, ['ID', 'Status']);
	}

	function getHiddens(){
		return we_html_element::hiddens(['home' => '0',
				'wcmd' => 'new_workflow',
				'wid' => $this->Model->ID,
				'pnt' => 'edit',
				'wname' => $this->uid,
				'page' => $this->page
		]);
	}

	function getHiddensFormPropertyPage(){
		$this->hiddens = array_merge($this->hiddens, ['Text', 'Type', 'Folders', 'ObjectFileFolders', 'Categories', 'ObjCategories', 'DocType', 'Objects', 'EmailPath',
			'LastStepAutoPublish']);

		return '';
	}

	function getHiddensFormOverviewPage(){
		//we need the following vars since fields expect this hidden fields & selectors don't generate a hidden field itself
		$this->hiddens = array_merge($this->hiddens, ['Folders', 'ObjectFileFolders', 'Categories', 'ObjCategories', 'Objects']);

		$out = '';

		$counter = 0;
		$counter1 = 0;
		foreach($this->Model->steps as $sv){
			$out .= we_html_element::hiddens([$this->uid . '_step' . $counter . '_sid' => $sv->ID,
					$this->uid . '_step' . $counter . '_and' => $sv->stepCondition,
					$this->uid . '_step' . $counter . '_Worktime' => $sv->Worktime,
					$this->uid . '_step' . $counter . '_timeAction' => $sv->timeAction
			]);
			$counter1 = 0;
			foreach($sv->tasks as $tv){
				$out .= we_html_element::hiddens([$this->uid . '_task' . $counter . $counter1 . '_tid' => $tv->ID,
						$this->uid . '_task_' . $counter . '_' . $counter1 . '_userid' => $tv->userID,
						$this->uid . '_task_' . $counter . '_' . $counter1 . '_Edit' => ($tv->Edit ? 1 : 0),
						$this->uid . '_task_' . $counter . '_' . $counter1 . '_Mail' => ($tv->Mail ? 1 : 0)
				]);
				++$counter1;
			}
			++$counter;
		}
		$out .= we_html_element::hiddens(['wcat' => '0',
				'wocat' => '0',
				'wfolder' => '0',
				'woffolder' => '0',
				'wobject' => '0',
				'wsteps' => $counter,
				'wtasks' => $counter1
		]);

		return $out;
	}

	function workflowHiddens(){
		$out = '';
		foreach($this->hiddens as $val){
			$dat = isset($this->Model->persistents[$val]) ? $this->Model->$val : $this->$val;
			$out .= we_html_element::hidden($this->uid . '_' . $val, (is_array($dat) ? implode(',', $dat) : $dat));
		}
		return $out;
	}

	function getProperties(we_base_jsCmd $jsCmd){
		$content = '<form name="we_form" onsubmit="return false">' .
			$this->getHiddens();
		if($this->show){
			$content .= $this->getDocumentInfo();
		} else {
			switch($this->page){
				case self::PAGE_PROPERTIES:
					$parts = [
						['headline' => g_l('modules_workflow', '[name]'),
							'html' => we_html_tools::textInput($this->uid . '_Text', stripslashes($this->Model->Text), '', ['id' => 'yuiAcInputPathName', 'onchange' => "top.content.setHot();",
								'onblur' => " parent.edheader.weTabs.setTitlePath(this.value);"], "text", 498),
							'icon' => we_html_multiIconBox::PROP_PATH
						],
						['headline' => g_l('modules_workflow', '[type]'),
							'html' => $this->getWorkflowTypeHTML($jsCmd),
							'icon' => we_html_multiIconBox::PROP_META
						],
						['headline' => g_l('modules_workflow', '[specials]'),
							'html' => '<br/>' .
							we_html_forms::checkboxWithHidden($this->Model->EmailPath, $this->uid . '_EmailPath', g_l('modules_workflow', '[EmailPath]'), false, 'defaultfont', '', false) .
							'<br/>' .
							we_html_forms::checkboxWithHidden($this->Model->LastStepAutoPublish, $this->uid . '_LastStepAutoPublish', g_l('modules_workflow', '[LastStepAutoPublish]'), false, 'defaultfont', '', false),
							'icon' => we_html_multiIconBox::PROP_ATTRIB
						],
					];
					//	Workflow-Type
					$content .= $this->getHiddensFormOverviewPage() .
						we_html_multiIconBox::getHTML('workflowProperties', $parts, true);
					break;
				case self::PAGE_OVERVIEW:
					$content .= $this->getHiddensFormPropertyPage() .
						we_html_tools::dialogLayout($this->getStepsHTML(), '');
			}
			$content .= $this->workflowHiddens();
		}
		$content .= '</form>';
		return we_html_element::body(['class' => 'weEditorBody', 'onload' => 'loaded=1;', 'onunload' => 'doUnload()'], $content);
	}

	function getWorkflowSelectHTML(){
		$vals = we_workflow_workflow::getAllWorkflowsInfo();
		return we_html_tools::select('wid', $vals, 4, $this->Model->ID, false, ['onclick' => "we_cmd(\"workflow_edit\")"], true, 200);
	}

	private function getWorkflowTypeHTML(we_base_jsCmd $jsCmd){
		return $this->getTypeTableHTML(we_html_forms::radiobutton(we_workflow_workflow::FOLDER, $this->Model->Type == we_workflow_workflow::FOLDER, $this->uid . '_Type', g_l('modules_workflow', '[type_dir]'), true, 'defaultfont', 'onclick=top.content.setHot();'), [
				$this->getFoldersHTML($jsCmd),
				], 25) .
			$this->getTypeTableHTML(we_html_forms::radiobutton(we_workflow_workflow::DOCTYPE_CATEGORY, $this->Model->Type == we_workflow_workflow::DOCTYPE_CATEGORY, $this->uid . '_Type', g_l('modules_workflow', '[type_doctype]'), true, 'defaultfont', 'onclick=top.content.setHot();'), [
				$this->getDocTypeHTML(),
				(we_modules_info::isActive(we_modules_info::CATEGORY) ? $this->getCategoryHTML($jsCmd) : ''),
				], 25) .
			(we_modules_info::isActive(we_modules_info::OBJECT) ?
			$this->getTypeTableHTML(we_html_forms::radiobutton(we_workflow_workflow::OBJECT, $this->Model->Type == we_workflow_workflow::OBJECT, $this->uid . '_Type', g_l('modules_workflow', '[type_object]'), true, 'defaultfont', 'onclick=top.content.setHot();'), [
				$this->getObjectHTML($jsCmd),
				(we_modules_info::isActive(we_modules_info::CATEGORY) ? $this->getObjCategoryHTML($jsCmd) : ''),
				$this->getObjectFileFoldersHTML($jsCmd),
				], 25) :
			'');
	}

	private function getFoldersHTML(we_base_jsCmd $jsCmd){
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:top.content.setHot();we_cmd('del_all_folders');");
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:top.content.setHot();we_cmd('we_selector_directory','',WE().consts.tables.FILE_TABLE,'','','add_folder','','','',true)");

		$dirs = new we_chooser_multiDir(495, $this->Model->Folders, 'del_folder', $delallbut . $addbut, '', 'ContentType', FILE_TABLE, 'defaultfont', '', "top.content.setHot();");

		return we_html_tools::formElementTable(g_l('modules_workflow', '[dirs]'), $dirs->get($jsCmd));
	}

	private function getCategoryHTML(we_base_jsCmd $jsCmd){
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:top.content.setHot();we_cmd('del_all_cats')", '', "", (!we_base_permission::hasPerm(we_base_perm::WORK_CATEGORY_EDIT)));
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:top.content.setHot();we_cmd('we_selector_document',0,WE().consts.tables.CATEGORY_TABLE,'','','add_cat')", '', "", (!we_base_permission::hasPerm(we_base_perm::WORK_CATEGORY_EDIT)));

		$cats = new we_chooser_multiDir(495, $this->Model->Categories, 'del_cat', $delallbut . $addbut, '', '"' . we_base_ContentTypes::CATEGORY, CATEGORY_TABLE . '"', 'defaultfont', '', "top.content.setHot();");

		return we_html_tools::formElementTable(g_l('modules_workflow', '[categories]'), $cats->get($jsCmd));
	}

	private function getObjCategoryHTML(we_base_jsCmd $jsCmd){
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:top.content.setHot();we_cmd('del_all_objcats')", '', "", (!we_base_permission::hasPerm(we_base_perm::WORK_CATEGORY_EDIT)));
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:top.content.setHot();we_cmd('we_selector_document',0,WE().consts.tables.CATEGORY_TABLE,'','','add_objcat')", '', "", (!we_base_permission::hasPerm(we_base_perm::WORK_CATEGORY_EDIT)));

		$cats = new we_chooser_multiDir(495, $this->Model->ObjCategories, 'del_objcat', $delallbut . $addbut, '', '"' . we_base_ContentTypes::CATEGORY . '"', CATEGORY_TABLE, 'defaultfont', '', "top.content.setHot();");

		return we_html_tools::formElementTable(g_l('modules_workflow', '[categories]'), $cats->get($jsCmd));
	}

	private function getObjectHTML(we_base_jsCmd $jsCmd){
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:top.content.setHot();we_cmd('del_all_objects')", '', "", (!we_base_permission::hasPerm(we_base_perm::WORK_CATEGORY_EDIT)));
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:top.content.setHot();we_cmd('openObjselector','',WE().consts.tables.OBJECT_TABLE,'','','add_object')", '', "", (!we_base_permission::hasPerm(we_base_perm::WORK_CATEGORY_EDIT)));

		$cats = new we_chooser_multiDir(495, $this->Model->Objects, 'del_object', $delallbut . $addbut, '', 'ContentType', OBJECT_TABLE, 'defaultfont', '', "top.content.setHot();");

		return we_html_tools::formElementTable(g_l('modules_workflow', '[classes]'), $cats->get($jsCmd));
	}

	private function getObjectFileFoldersHTML(we_base_jsCmd $jsCmd){
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:top.content.setHot();we_cmd('del_all_object_file_folders');");
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:top.content.setHot();we_cmd('we_selector_directory','',WE().consts.tables.OBJECT_FILES_TABLE,'','','add_object_file_folder','','','',true);");

		$dirs = new we_chooser_multiDir(495, $this->Model->ObjectFileFolders, 'del_object_file_folder', $delallbut . $addbut, '', 'ContentType', OBJECT_FILES_TABLE, 'defaultfont', '', "top.content.setHot();");

		return we_html_tools::formElementTable(g_l('modules_workflow', '[dirs]'), $dirs->get($jsCmd));
	}

	function getStatusHTML(){
		return we_html_forms::checkboxWithHidden(1, 'status_workflow', g_l('modules_workflow', '[active]'), false, 'defaultfont', 'top.content.setHot();');
	}

	function getStepsHTML(){
		$content = [];

		$ids = '';

		$headline = ['<div class="middlefont">' . g_l('modules_workflow', '[step]') . '</div>',
			'<div class="middlefont">' . g_l('modules_workflow', '[and_or]') . '</div>',
			'<div class="middlefont">' . g_l('modules_workflow', '[worktime]') . '</div>'
		];

		$counter = 0;
		$counter1 = 0;

		$weSuggest = & we_gui_suggest::getInstance();

		/*		 * *** WORKFLOWSTEPS **** */
		foreach($this->Model->steps as $sv){
			$ids .= we_html_element::hidden($this->uid . '_step' . $counter . '_sid', $sv->ID);
			$content[$counter] = [['dat' => $counter + 1,
				'height' => '',
				'align' => 'center',
				],
				['dat' => '<table><tr style="vertical-align:top"><td>' . we_html_forms::radiobutton(1, $sv->stepCondition ? 1 : 0, $this->uid . "_step" . $counter . "_and", "", false, "defaultfont", "top.content.setHot();") . '</td><td style="padding-left:5px;">' . we_html_forms::radiobutton(0, $sv->stepCondition ? 0 : 1, $this->uid . "_step" . $counter . "_and", "", false, "defaultfont", "top.content.setHot();") . '</td></tr></table>',
					'height' => '',
					'align' => '',
				],
				['dat' => '<table class="default" style="margin-top:5px;">
	<tr style="vertical-align:middle"><td class="middlefont" style="padding-bottom:1em;">' . we_html_tools::textInput($this->uid . "_step" . $counter . "_Worktime", $sv->Worktime, '', [
						'min' => "0", 'step' => "0.016", 'onchange' => "top.content.setHot();"], 'number') . '</td></tr>
	<tr style="vertical-align:top"><td class="middlefont">' . we_html_forms::checkboxWithHidden($sv->timeAction == 1, $this->uid . "_step" . $counter . "_timeAction", g_l('modules_workflow', '[go_next]'), false, 'middlefont', "top.content.setHot();") . '</td></tr>
</table>',
					'height' => '',
					'align' => '',
				]
			];


			$counter1 = 0;
			foreach($sv->tasks as $tv){
				$ids .= we_html_element::hidden($this->uid . '_task' . $counter . '_' . $counter1 . '_tid', $tv->ID);
				$headline[$counter1 + 3] = g_l('modules_workflow', '[user]') . (string) ($counter1 + 1);

				$foo = f('SELECT Path FROM ' . USER_TABLE . ' WHERE ID=' . intval($tv->userID), '', $this->db);

				$weSuggest->setAcId('User_' . $counter . '_' . $counter1);
				$weSuggest->setContentType([we_users_user::TYPE_USER, we_users_user::TYPE_USER_GROUP]);
				$weSuggest->setInput($this->uid . '_task_' . $counter . '_' . $counter1 . '_usertext', $foo, [], false, true);
				$weSuggest->setMaxResults(10);
				$weSuggest->setRequired(true);
				$weSuggest->setResult($this->uid . '_task_' . $counter . '_' . $counter1 . '_userid', $tv->userID);
				$weSuggest->setSelector(we_gui_suggest::DocSelector);
				$weSuggest->setTable(USER_TABLE);
				$weSuggest->setWidth(200);
				$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:top.content.setHot();we_cmd('we_users_selector',document.we_form." . $this->uid . '_task_' . $counter . '_' . $counter1 . "_userid.value, '" . $this->uid . "_task_" . $counter . "_" . $counter1 . "_usertext','','" . $this->uid . "_task_" . $counter . "_" . $counter1 . "_userid');"));

				$content[$counter][$counter1 + 3] = ['dat' => '<table class="default" style="margin-top:1ex;">
						<tr style="vertical-align:middle"><td>' . $weSuggest->getHTML() . '</td>
						</tr></table>
						<table class="default">
						<tr style="vertical-align:top">
						<td class="middlefont" style="text-align:right">' . we_html_forms::checkboxWithHidden($tv->Mail, $this->uid . "_task_" . $counter . "_" . $counter1 . "_Mail", g_l('modules_workflow', '[send_mail]'), false, 'middlefont', "top.content.setHot();") . '</td>
						<td class="middlefont" style="padding-left:20px;">' . we_html_forms::checkboxWithHidden($tv->Edit, $this->uid . "_task_" . $counter . "_" . $counter1 . "_Edit", g_l('modules_workflow', '[edit]'), false, 'middlefont', "top.content.setHot();") . '</td>
						</tr></table>',
					'height' => '',
					'align' => ''
				];
				$counter1++;
			}
			++$counter;
		}
		return $ids . '
<table style="margin-right:30px;">
	<tr style="vertical-align:top">
		<td>' . we_html_tools::dialogBorder(400, $content, $headline) . '</td>
		<td><table class="default" style="margin-top:3px;">
			<tr><td>' . we_html_button::create_button(we_html_button::PLUS, "javascript:top.content.setHot();addTask()") . we_html_button::create_button(we_html_button::TRASH, "javascript:top.content.setHot();delTask()") . '</td>
			</tr>
			</table></td>
	</tr>
	<tr style="vertical-align:top">
		<td colspan="2">' . we_html_button::create_button(we_html_button::PLUS, "javascript:top.content.setHot();addStep()") . we_html_button::create_button(we_html_button::TRASH, "javascript:top.content.setHot();delStep()") . '</td></tr>
</table>' .
			we_html_element::hiddens(['wsteps' => $counter,
				'wtasks' => $counter1
		]);
	}

	function getTypeTableHTML($head, $values, $ident = 0, $textalign = "left", $textclass = "defaultfont"){
		$out = '<table class="default">' . ($head ? '<tr><td class="' . trim($textclass) . '" style="text-align:' . trim($textalign) . '" colspan="2">' . $head . '</td></tr>' : '');
		foreach($values as $val){
			$out .= '<tr><td class="' . trim($textclass) . '" style="padding-left:' . $ident . 'px;">' . $val . '</td></tr>';
		}
		$out .= '</table>';
		return $out;
	}

	private function getDocTypeHTML(){
		$dtq = we_doctype_docTypes::getDoctypeQuery($this->db);
		$vals = $this->db->getAllFirstq('SELECT dt.ID,dt.DocType FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE ' . $dtq['where'], false);
		return we_html_tools::formElementTable(g_l('modules_workflow', '[doctype]'), we_html_tools::select($this->uid . '_DocType[]', $vals, 1, $this->Model->DocType, true, [
					'class' => 'searchSelect',
					'onchange' => "top.content.setHot();"], true, 498));
	}

	function getJSTop(){
		$modData = we_modules_info::getModuleField(we_modules_info::WORKFLOW, 'text');
		$title = ($modData ? 'webEdition ' . g_l('global', '[modules]') . ' - ' . $modData : '');
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'workflow/workflow_top.js', "parent.document.title='" . $title . "';");
	}

	function getJSProperty(array $jsVars = []){
		if(!$this->show){
			$this->Model->loadDocuments();
		}
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'workflow/workflow_property.js', '', ['id' => 'loadVarWorkflowProp', 'data-workflow' => setDynamicVar(array_merge($jsVars, [
					'uid' => $this->uid,
					'noDocs' => ($this->show ? 0 : count($this->Model->documents))
		]))]);
	}

	public function processCommands(we_base_jsCmd $jscmd){
		switch(we_base_request::_(we_base_request::STRING, 'wcmd', '')){
			case 'new_workflow':
				$this->Model = new we_workflow_workflow();
				$this->page = self::PAGE_PROPERTIES;

				$jscmd->addCmd('loadHeadFooter', ['page' => self::PAGE_PROPERTIES, 'txt' => '']);
				break;
			case 'add_cat':
				$arr = $this->Model->Categories;
				if(($ids = we_base_request::_(we_base_request::INTLISTA, 'wcat', []))){
					$this->Model->Categories = array_unique(array_filter(array_merge($arr, $ids)), SORT_NUMERIC);
				}
				break;
			case 'del_cat':
				$arr = $this->Model->Categories;
				if(($cat = we_base_request::_(we_base_request::INT, 'wcat'))){
					if(($pos = array_search($cat, $arr, false)) === false){
						break;
					}
					unset($arr[$pos]);
					$this->Model->Categories = $arr;
				}
				break;
			case 'del_all_cats':
				$this->Model->Categories = [];
				break;
			case 'add_objcat':
				$arr = $this->Model->ObjCategories;
				if(($ids = we_base_request::_(we_base_request::INTLISTA, 'wocat', []))){
					$this->Model->ObjCategories = array_unique(array_filter(array_merge($arr, $ids)), SORT_NUMERIC);
				}
				break;
			case 'del_objcat':
				$arr = $this->Model->ObjCategories;
				if(($cat = we_base_request::_(we_base_request::INT, 'wcat'))){
					if(($pos = array_search($cat, $arr, false)) === false){
						break;
					}
					unset($arr[$pos]);
					$this->Model->ObjCategories = $arr;
				}
				break;
			case 'del_all_objcats':
				$this->Model->ObjCategories = [];
				break;
			case 'add_folder':
				$arr = $this->Model->Folders;
				if(($ids = we_base_request::_(we_base_request::INTLISTA, 'wfolder')) !== false){
					$this->Model->Folders = array_unique(array_filter(array_merge($arr, $ids)), SORT_NUMERIC);
					;
				}
				break;
			case 'del_folder':
				$arr = $this->Model->Folders;
				if(($id = we_base_request::_(we_base_request::INT, 'wfolder')) !== false){
					if(($pos = array_search($id, $arr, false)) === false){
						break;
					}
					unset($arr[$pos]);
					$this->Model->Folders = $arr;
				}
				break;
			case 'del_all_folders':
				$this->Model->Folders = [];
				break;
			case 'add_object_file_folder':
				$arr = $this->Model->ObjectFileFolders;
				if(($ids = we_base_request::_(we_base_request::INTLISTA, 'woffolder')) !== false){
					$this->Model->ObjectFileFolders = array_unique(array_filter(array_merge($arr, $ids)), SORT_NUMERIC);
					;
				}
				break;
			case 'del_object_file_folder':
				$arr = $this->Model->ObjectFileFolders;
				if(($id = we_base_request::_(we_base_request::INT, 'woffolder')) !== false){
					if(($pos = array_search($id, $arr, false)) === false){
						break;
					}
					unset($arr[$pos]);
					$this->Model->ObjectFileFolders = $arr;
				}
				break;
			case 'del_all_object_file_folders':
				$this->Model->ObjectFileFolders = [];
				break;
			case 'add_object':
				$arr = $this->Model->Objects;
				if(($id = we_base_request::_(we_base_request::INT, 'wobject'))){
					$arr[] = $id;
					$this->Model->Objects = $arr;
				}
				break;
			case 'del_object':
				$arr = $this->Model->Objects;
				if(($id = we_base_request::_(we_base_request::INT, 'wobject'))){
					if(($pos = array_search($id, $arr, false)) === false){
						break;
					}
					unset($arr[$pos]);
					$this->Model->Objects = $arr;
				}
				break;
			case 'del_all_objects':
				$this->Model->Objects = [];
				break;
			case 'reload':
				$jscmd->addCmd('loadHeadFooter', ['page' => $this->page, 'txt' => $this->Model->Text]);
				break;
			case 'workflow_edit':
				$this->show = 0;
				if(($id = we_base_request::_(we_base_request::INT, 'wid'))){
					$this->Model = new we_workflow_workflow($id);
				}

				$_REQUEST['wcmd'] = 'reload';
				$this->processCommands($jscmd);
				break;
			case 'switchPage':
				if(($page = we_base_request::_(we_base_request::INT, 'page')) !== false){
					$this->page = $page;
				}
				break;
			case 'save_workflow':
				if(we_base_request::_(we_base_request::INT, 'wid') !== false){
					$newone = (!$this->Model->ID);

					if(!we_base_permission::hasPerm([we_base_perm::WORKFLOW_EDIT, we_base_perm::WORKFLOW_NEW])){
						$jscmd->addMsg(g_l('modules_workflow', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
						return;
					}
					if($newone && !we_base_permission::hasPerm(we_base_perm::WORKFLOW_NEW)){
						$jscmd->addMsg(g_l('modules_workflow', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
						return;
					}
					//duplicate
					if(f('SELECT 1 FROM ' . WORKFLOW_TABLE . ' WHERE Text="' . $this->db->escape($this->Model->Text) . '"' . ($newone ? '' : ' AND ID!=' . intval($this->Model->ID)) . ' LIMIT 1', '', $this->db)){
						$jscmd->addMsg(g_l('modules_workflow', '[double_name]'), we_base_util::WE_MESSAGE_ERROR);
						return;
					}
					$this->Model->loadDocuments();
					foreach($this->Model->documents as $v){
						$jscmd->addCmd('deleteTreeEntry', [$v['ID'], 'file']);
					}
					if(($dts = we_base_request::_(we_base_request::INTLISTA, $this->uid . '_DocType')) !== false){
						$this->Model->DocType = $dts;
					}

					$this->Model->save();
					if($newone){
						$jscmd->addCmd('makeTreeEntry', [
							'id' => $this->Model->ID,
							'parentid' => 0,
							'text' => $this->Model->Text,
							'open' => true,
							'contenttype' => we_base_ContentTypes::FOLDER,
							'table' => "we_workflow_workflowDef",
							'published' => $this->Model->Status
						]);
					} else {
						$jscmd->addCmd('updateTreeEntry', [
							'id' => $this->Model->ID,
							'text' => $this->Model->Text,
							'published' => $this->Model->Status
						]);
					}
					$jscmd->addCmd('setHeadRow', we_html_element::b(g_l('modules_workflow', '[workflow]') . ': ' . oldHtmlspecialchars($this->Model->Text)));
					$jscmd->addMsg(g_l('modules_workflow', '[save_ok]'), we_base_util::WE_MESSAGE_NOTICE);
				}
				break;
			case 'show_document':
				if(($id = we_base_request::_(we_base_request::INT, 'wid'))){
					$this->show = 1;
					$this->page = self::PAGE_PROPERTIES;
					$this->documentDef->load($id);
					$jscmd->addCmd('loadHeadFooter', ['page' => self::PAGE_PROPERTIES, 'txt' => $this->documentDef->document->Text, 'art' => 1]);
				}
				break;
			case 'delete_workflow':
				if(($id = we_base_request::_(we_base_request::INT, 'wid'))){
					if(!we_base_permission::hasPerm(we_base_perm::WORKFLOW_DELETE)){
						$jscmd->addMsg(g_l('modules_workflow', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
						return;
					}

					$this->Model = new we_workflow_workflow($id);
					if($this->Model->delete()){
						$this->Model = new we_workflow_workflow();
						$jscmd->addCmd('deleteTreeEntry', [$id, we_base_ContentTypes::FOLDER]);
						$jscmd->addMsg(g_l('modules_workflow', '[delete_ok]'), we_base_util::WE_MESSAGE_NOTICE);
					} else {
						$jscmd->addMsg(g_l('modules_workflow', '[delete_nok]'), we_base_util::WE_MESSAGE_ERROR);
					}
				}
				break;
			case 'reload_table':
				$this->page = self::PAGE_OVERVIEW;
				break;
			case 'empty_log':
				$stamp = 0;
				if(($t = we_base_request::_(we_base_request::INTLISTA, 'wopt', []))){
					$stamp = mktime($t[3], $t[4], 0, $t[1], $t[0], $t[2]);
				}
				$this->Log->clearLog($stamp);
				$jscmd->addMsg(g_l('modules_workflow', '[empty_log_ok]'), we_base_util::WE_MESSAGE_NOTICE);
				break;
			default:
		}
	}

	public function processVariables(){
		$this->uid = we_base_request::_(we_base_request::STRING, 'wname', $this->uid);
		foreach($this->Model->persistents as $val => $type){
			$varname = $this->uid . '_' . $val;
			if(($tmp = we_base_request::_($type, $varname, '_noval_')) !== '_noval_'){
				$this->Model->$val = $tmp;
			}
		}

		$wsteps = we_base_request::_(we_base_request::INT, 'wsteps', 0);
		$wtasks = we_base_request::_(we_base_request::INT, 'wtasks', 0);
		$this->page = we_base_request::_(we_base_request::INT, 'page', self::PAGE_PROPERTIES);


		$this->Model->steps = [];
		if($wsteps == 0){
			$this->Model->addNewStep();
			$this->Model->addNewTask();
		}

		for($i = 0; $i < $wsteps; $i++){
			$this->Model->addNewStep();
		}
		for($j = 0; $j < $wtasks; $j++){
			$this->Model->addNewTask();
		}

		foreach($this->Model->steps as $skey => &$step){
			$step->workflowID = $this->Model->ID;

			if(($tmp = we_base_request::_(we_base_request::INT, $this->uid . '_step' . $skey . '_sid'))){
				$step->ID = $tmp;
			}

			$step->stepCondition = we_base_request::_(we_base_request::BOOL, $this->uid . '_step' . $skey . '_and', $step->stepCondition);
			$step->Worktime = we_base_request::_(we_base_request::FLOAT, $this->uid . '_step' . $skey . '_Worktime', $step->Worktime);
			$step->timeAction = we_base_request::_(we_base_request::BOOL, $this->uid . '_step' . $skey . '_timeAction', $step->timeAction);

			foreach($step->tasks as $tkey => &$task){
				$task->stepID = $this->Model->steps[$skey]->ID;
				if(($id = we_base_request::_(we_base_request::INT, $this->uid . '_task_' . $skey . '_' . $tkey . '_tid'))){
					$task->ID = $id;
				}

				$task->userID = we_base_request::_(we_base_request::INT, $this->uid . '_task_' . $skey . '_' . $tkey . '_userid', $task->userID);
				$task->username = we_base_request::_(we_base_request::STRING, $this->uid . '_task_' . $skey . '_' . $tkey . '_usertext', $task->username); //FIXME: this is a path to a user
				$task->Edit = we_base_request::_(we_base_request::BOOL, $this->uid . '_task_' . $skey . '_' . $tkey . '_Edit', $task->Edit);
				$task->Mail = we_base_request::_(we_base_request::BOOL, $this->uid . '_task_' . $skey . '_' . $tkey . '_Mail', $task->Mail);
			}
		}
	}

	private function getDocumentInfo(){
		if($this->documentDef->workflow->Type == we_workflow_workflow::OBJECT){
			return $this->getObjectInfo();
		}

		//	Part - file-information
		$parts = [['headline' => g_l('weEditorInfo', '[content_type]'),
			'html' => g_l('weEditorInfo', '[' . $this->documentDef->document->ContentType . ']'),
			'space' => we_html_multiIconBox::SPACE_MED,
			'noline' => (($this->documentDef->document->ContentType != we_base_ContentTypes::FOLDER && $this->documentDef->workflow->Type != we_workflow_workflow::OBJECT) ? 1 : 0)
			]
		];
		if($this->documentDef->document->ContentType != we_base_ContentTypes::FOLDER && $this->documentDef->workflow->Type != we_workflow_workflow::OBJECT){
			$GLOBALS['we_doc'] = $this->documentDef->document;
			$fs = $this->documentDef->document->getFilesize($this->documentDef->document->Path);
			$parts[] = ['headline' => g_l('weEditorInfo', '[file_size]'),
				'html' => we_base_file::getHumanFileSize($fs) . '&nbsp;KB&nbsp;(' . we_base_file::getHumanFileSize($fs, we_base_file::SZ_BYTE) . ')',
				'space' => we_html_multiIconBox::SPACE_MED
			];
		}

		//	Part - publish-information
		$parts[] = ['headline' => g_l('weEditorInfo', '[creation_date]'),
			'html' => date(g_l('weEditorInfo', '[date_format]'), $this->documentDef->document->CreationDate),
			'space' => we_html_multiIconBox::SPACE_MED,
			'noline' => true
		];

		if($this->documentDef->document->CreatorID){
			$creator = f('SELECT CONCAT(First," ",Second," (",username,")") FROM ' . USER_TABLE . ' WHERE ID=' . $this->documentDef->document->CreatorID);
			if($creator){
				$parts[] = ['headline' => g_l('modules_users', '[created_by]'),
					'html' => $creator,
					'space' => we_html_multiIconBox::SPACE_MED,
					'noline' => true
				];
			}
		}

		$parts[] = ['headline' => g_l('weEditorInfo', '[changed_date]'),
			'html' => date(g_l('weEditorInfo', '[date_format]'), $this->documentDef->document->ModDate),
			'space' => we_html_multiIconBox::SPACE_MED,
			'noline' => true
		];

		if($this->documentDef->document->ModifierID){
			$modifier = f('SELECT CONCAT(First," ",Second," (",username,")") FROM ' . USER_TABLE . ' WHERE ID=' . $this->documentDef->document->ModifierID);
			if($modifier){
				$parts[] = ['headline' => g_l('modules_users', '[changed_by]'),
					'html' => $modifier,
					'space' => we_html_multiIconBox::SPACE_MED,
					'noline' => true
				];
			}
		}

		if($this->documentDef->document->ContentType == we_base_ContentTypes::HTML || $this->documentDef->document->ContentType == we_base_ContentTypes::WEDOCUMENT){
			$parts[] = ['headline' => g_l('weEditorInfo', '[lastLive]'),
				'html' => ($this->documentDef->document->Published ? date(g_l('weEditorInfo', '[date_format]'), $this->documentDef->document->Published) : '-'),
				'space' => we_html_multiIconBox::SPACE_MED
			];
		}

		//	Part - Path-information

		if($this->documentDef->document->Table !== TEMPLATES_TABLE && $this->documentDef->workflow->Type != we_workflow_workflow::OBJECT){
			$rp = realpath($this->documentDef->document->getRealPath());
			$http = $this->documentDef->document->getHttpPath();

			switch($this->documentDef->document->ContentType){
				default:
					$showlink = false;
					break;
				case we_base_ContentTypes::HTML:
				case we_base_ContentTypes::WEDOCUMENT:
				case we_base_ContentTypes::IMAGE:
				case we_base_ContentTypes::FLASH:
				case we_base_ContentTypes::VIDEO:
				case we_base_ContentTypes::AUDIO:
					$showlink = true;
			}

			$parts[] = ['headline' => g_l('weEditorInfo', '[local_path]'),
				'html' => '<a href="#" style="text-decoration:none;cursor:text" class="defaultfont" title="' . $rp . '" onclick="WE().layout.openToEdit(\'' . $this->documentDef->document->Table . '\',\'' . $this->documentDef->document->ID . '\',\'' . $this->documentDef->document->ContentType . '\')" >' . we_base_util::shortenPath($rp, 74) . '</a>',
				'space' => we_html_multiIconBox::SPACE_MED,
				'noline' => true
			];

			$parts[] = ['headline' => g_l('weEditorInfo', '[http_path]'),
				'html' => ($showlink ? '<a href="' . $http . '" target="_blank" title="' . $http . '">' : '') . we_base_util::shortenPath($http, 74) . ($showlink ? '</a>' : ''),
				'space' => we_html_multiIconBox::SPACE_MED
			];
			$parts[] = ['headline' => '',
				'html' => '<a href="#" onclick="WE().layout.openToEdit(\'' . $this->documentDef->document->Table . '\',\'' . $this->documentDef->document->ID . '\',\'' . $this->documentDef->document->ContentType . '\')" >' . g_l('weEditorInfo', '[openDocument]') . '</a>',
				'space' => we_html_multiIconBox::SPACE_MED
			];
		}

		//	Logbook
		$parts[] = ['headline' => '',
			'html' => self::getDocumentStatus($this->documentDef->ID),
		];

		return we_html_element::jsScript(JS_DIR . 'tooltip.js') .
			we_html_multiIconBox::getHTML('', $parts, true);
	}

	private function getObjectInfo(){
		//	Dokument properties
		$parts = [['headline' => 'ID',
			'html' => $this->documentDef->document->ID,
			'space' => we_html_multiIconBox::SPACE_MED2,
			'noline' => true
			],
			['headline' => g_l('weEditorInfo', '[content_type]'),
				'html' => g_l('weEditorInfo', '[' . $this->documentDef->document->ContentType . ']'),
				'space' => we_html_multiIconBox::SPACE_MED2,
			],
			// publish information
			['headline' => g_l('weEditorInfo', '[creation_date]'),
				'html' => date(g_l('weEditorInfo', '[date_format]'), $this->documentDef->document->CreationDate),
				'space' => we_html_multiIconBox::SPACE_MED2,
				'noline' => true
			],
			(($user = f('SELECT CONCAT(First," ",Second," (",username,")") FROM ' . USER_TABLE . ' WHERE ID=' . $this->documentDef->document->CreatorID)) ?
			['headline' => g_l('modules_users', '[created_by]'),
			'html' => $user,
			'space' => we_html_multiIconBox::SPACE_MED2,
			'noline' => true
			] : []),
			['headline' => g_l('weEditorInfo', '[changed_date]'),
				'html' => date(g_l('weEditorInfo', '[date_format]'), $this->documentDef->document->ModDate),
				'space' => we_html_multiIconBox::SPACE_MED2,
				'noline' => true
			],
			(($user = f('SELECT CONCAT(First," ",Second," (",username,")") FROM ' . USER_TABLE . ' WHERE ID=' . $this->documentDef->document->ModifierID)) ?
			['headline' => g_l('modules_users', '[changed_by]'),
			'html' => $user,
			'space' => we_html_multiIconBox::SPACE_MED2,
			'noline' => true
			] : []),
			['headline' => g_l('weEditorInfo', '[lastLive]'),
				'html' => ($this->documentDef->document->Published ? date(g_l('weEditorInfo', '[date_format]'), $this->documentDef->document->Published) : '-'),
				'space' => we_html_multiIconBox::SPACE_MED2,
			],
			['headline' => '',
				'html' => '<a href="#" onclick="WE().layout.openToEdit(\'' . $this->documentDef->document->Table . '\',\'' . $this->documentDef->document->ID . '\',\'' . $this->documentDef->document->ContentType . '\')" >' . g_l('weEditorInfo', '[openDocument]') . '</a>',
				'space' => we_html_multiIconBox::SPACE_MED2
			],
			['headline' => '',
				'html' => self::getDocumentStatus($this->documentDef->ID),
			]
		];

		return we_editor_script::get() .
			we_html_element::jsScript(JS_DIR . 'global.js', 'initWE();') .
			'</head>
		<body class="weEditorBody" onunload="doUnload()">
				<form name="we_form">' . we_contents_base::hiddenTrans() . '<table>' .
			we_html_multiIconBox::getHTML('', $parts, true) .
			'</form></body></html>';
	}

	private static function getTime($seconds){
		$min = floor($seconds / 60);
		$ret = ['hour' => floor($min / 60),
			'min' => $min,
			'sec' => $seconds - ($min * 60)];
		$ret['min'] -= ($ret['hour'] * 60);
		return $ret;
	}

	public static function getDocumentStatus($workflowDocID){
		$db = new DB_WE();
		$headline = ['<div class="middlefont">' . g_l('modules_workflow', '[step]') . '</div>'];

		$workflowDocument = new we_workflow_document($workflowDocID);

		$counter = 0;
		$counter1 = 0;
		$current = $workflowDocument->findLastActiveStep();
		if($current < 0){
			return g_l('modules_workflow', '[cannot_find_active_step]');
		}
		foreach($workflowDocument->steps as $sk => $sv){

			$workflowStep = new we_workflow_step($sv->workflowStepID);

			/* $now = date(g_l('weEditorInfo', '[date_format]'), time());
			  $start = date(g_l('weEditorInfo', '[date_format]'), $sv->startDate);
			 */
			$elapsed = self::getTime(time() - $sv->startDate);
			$remained = self::getTime(($sv->startDate + round($workflowStep->Worktime * 3600)) - time());

			if($remained['hour'] < 0){
				if($sk > $current){
					$finished_font = 'middlefont';
					$notfinished_font = 'middlefont lowContrast';
				} else {
					$finished_font = 'middlefont highlightElementChanged';
					$notfinished_font = 'middlefont highlightElementChanged';
				}
			} else {
				$finished_font = 'middlefont';
				$notfinished_font = 'middlefont lowContrast';
			}

			$end = date(g_l('weEditorInfo', '[date_format]'), $sv->startDate + round($workflowStep->Worktime * 3600));

			$content[$counter] = [['dat' => ($sv->Status == we_workflow_documentStep::STATUS_NEW ? '<div class="' . $notfinished_font . '">' : '<div class="' . $finished_font . '">') . ($counter + 1) . "</div>",
				'height' => '',
				'align' => 'center'
				]
			];

			$counter1 = 0;
			foreach($sv->tasks as $tv){

				$headline[++$counter1] = g_l('modules_workflow', '[user]') . (string) ($counter1);

				$workflowTask = new we_workflow_task($tv->workflowTaskID);

				$foo = f('SELECT username FROM ' . USER_TABLE . ' WHERE ID=' . intval($workflowTask->userID), '', $db);

				if($sk == $current){
					$out = ($tv->Status == we_workflow_documentTask::STATUS_NEW ? '<div class="' . $notfinished_font . '">' : '<div class="' . $finished_font . '">') . $foo . "</div>";
				} else if($sk < $current){
					$out = '<div class="' . $finished_font . '">' . $foo . '</div>';
				} else {
					$out = '<div class="' . $notfinished_font . '">' . $foo . '</div>';
				}

				$content[$counter][$counter1] = ['dat' => $out,
					'height' => '',
					'align' => '',
				];
			}


			$headline[++$counter1] = g_l('modules_workflow', '[worktime]');

			$content[$counter][$counter1] = ['dat' => ($sv->Status == we_workflow_documentStep::STATUS_NEW ? '<div class="' . $notfinished_font . '">' : '<div class="' . $finished_font . '">') . $workflowStep->Worktime . '</div>',
				'height' => '',
				'align' => 'right'];

			if($sk <= $current){
				$headline[++$counter1] = g_l('modules_workflow', '[time_elapsed]');


				$content[$counter][$counter1] = ['dat' => ($sv->Status == we_workflow_documentStep::STATUS_NEW ? '<div class="' . $notfinished_font . '">' : '<div class="' . $finished_font . '">') . $elapsed['hour'] . ":" . $elapsed['min'] . ":" . $elapsed['sec'] . "</div>",
					'height' => '',
					'align' => 'right',
				];

				$headline[++$counter1] = g_l('modules_workflow', '[time_remained]');

				$content[$counter][$counter1] = ['dat' => ($sv->Status == we_workflow_documentStep::STATUS_NEW ? '<div class="' . $notfinished_font . '">' : '<div class="' . $finished_font . '">') . $remained['hour'] . ":" . $remained['min'] . ":" . $remained['sec'] . "</div>",
					'height' => '',
					'align' => 'right',
				];

				$headline[++$counter1] = g_l('modules_workflow', '[step_plan]');

				$content[$counter][$counter1] = ['dat' => ($sv->Status == we_workflow_documentStep::STATUS_NEW ? '<div class="' . $notfinished_font . '">' : '<div class="' . $finished_font . '">') . $end . "</div>",
					'height' => '',
					'align' => 'right',
				];
			}
			++$counter;
		}

		$wfType = f('SELECT ' . WORKFLOW_TABLE . '.Type FROM ' . WORKFLOW_TABLE . ',' . WORKFLOW_DOC_TABLE . ' WHERE ' . WORKFLOW_DOC_TABLE . '.workflowID=' . WORKFLOW_TABLE . '.ID AND ' . WORKFLOW_DOC_TABLE . '.ID=' . intval($workflowDocument->ID), '', $db);
		return '<table class="default" style="margin-right:15px;">
		<tr><td>' . we_html_tools::dialogBorder(730, $content, $headline) . '</td></tr>
		<tr><td style="padding-top:10px;">' . we_html_button::create_button(we_html_button::LOG_SHOW, "javascript:new (WE().util.jsWindow)(window, '" . WEBEDITION_DIR . 'we_showMod.php?mod=wrokflow&pnt=log&art=' . $workflowDocument->document->ID . "&type=" . $wfType . "','workflow_history',WE().consts.size.dialog.medium,WE().consts.size.dialog.small,true,false,true);") . '</td></tr>		</table>';
	}

	static function getLogForDocument($docID, $type = 0){//type is an string-array
		$content = [];

		$headlines = [['dat' => g_l('modules_workflow', '[action]')],
			['dat' => g_l('modules_workflow', '[description]')],
			['dat' => g_l('modules_workflow', '[time]')],
			['dat' => g_l('modules_workflow', '[user]')],
		];

		$logs = we_workflow_log::getLogForDocument($docID, 'DESC', $type);
		$offset = we_base_request::_(we_base_request::INT, 'offset', 0);
		$art = we_base_request::_(we_base_request::INT, 'art', '');
		$numRows = we_workflow_log::NUMBER_LOGS;
		$anz = $GLOBALS['ANZ_LOGS'];

		foreach($logs as $v){
			$content[] = [['dat' => '<div class="middlefont">' . $v['Event'] . '</div>',
				'height' => '',
				'align' => '',
				],
				['dat' => '<div class="middlefont">' . $v['Description'] . '</div>',
					'height' => '',
					'align' => '',
				],
				['dat' => '<div class="middlefont">' . $v['Datum'] . '</div>',
					'height' => '',
					'align' => 'right',
				],
				['dat' => '<div class="middlefont">' . ((!empty($foo['First'])) ? $foo['First'] : '-') . ' ' . ((!empty($foo['Second'])) ? $foo['Second'] : '-') . ((!empty($foo['username'])) ? ' (' . $foo['username'] . ')' : '') . '</div>',
					'height' => '',
					'align' => 'left',
				],
			];
		}

		$nextprev = '<table class="default"><tr><td>' .
			($offset ?
			we_html_button::create_button(we_html_button::BACK, WEBEDITION_DIR . 'we_showMod.php?mod=workflow&pnt=log&art=' . $art . '&type=' . $type . '&offset=' . ($offset - $numRows)) :
			we_html_button::create_button(we_html_button::BACK, '', '', '', true)
			) .
			'</td><td class="defaultfont" style="padding: 0 10px 0 10px;"><b>' . (($anz) ? $offset + 1 : 0) . '-' .
			(($anz - $offset) < $numRows ? $anz : $offset + $numRows) . ' ' . g_l('global', '[from]') . ' ' . $anz . '</b></td><td>' .
			((($offset + $numRows) < $anz) ?
			we_html_button::create_button(we_html_button::NEXT, WEBEDITION_DIR . 'we_showMod.php?mod=workflow&pnt=log&art=' . $art . '&type=' . $type . '&offset=' . ($offset + $numRows)/* . "&order=$order" */) :
			we_html_button::create_button(we_html_button::NEXT, '', '', '', true)
			) .
			'</td><td></tr></table>';

		$buttonsTable = '<table style="width:100%" class="default"><tr><td>' . $nextprev . '</td><td style="text-align:right">' . we_html_button::create_button(we_html_button::CLOSE, "javascript:self.close();") . '</td></tr></table>';


		return ($logs ?
			we_html_tools::dialogLayout(we_html_tools::dialogBorder(580, $content, $headlines), '', $buttonsTable) :
			we_html_tools::dialogLayout('<div style="width:500px;text-align:center" class="middlefont">-- ' . g_l('modules_workflow', '[log_is_empty]') . ' --</div>', '', we_html_button::create_button(we_html_button::CLOSE, "javascript:self.close();")));
	}

	function getLogQuestion(){
		$vals = ['<table class="default" style="margin-left:22px;"><tr><td>' . we_html_tools::getDateInput("log_time%s", (time() - (336 * 3600))) . '</td></tr></table>'];

		return we_html_tools::dialogLayout(
				we_html_element::hidden('clear_opt', 1) .
				'<form name="we_form">' .
				'<table class="default">' .
				'<tr><td class="defaultfont" style="padding-bottom:10px;">' . g_l('modules_workflow', '[log_question_text]') . '</td></tr>' .
				'<tr><td>' . $this->getTypeTableHTML(we_html_forms::radiobutton(1, true, 'clear_time', g_l('modules_workflow', '[log_question_time]'), true, 'defaultfont', "javascript:document.we_form.clear_opt.value=1;"), $vals) . '</td></tr>' .
				'<tr><td style="padding-top:1em;">' . we_html_forms::radiobutton(0, false, 'clear_time', g_l('modules_workflow', '[log_question_all]'), true, 'defaultfont', "javascript:document.we_form.clear_opt.value=0;") . '</td></tr>' .
				'</table>'
				, g_l('modules_workflow', '[empty_log]'), we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, 'javascript:self.clearLog();'), '', we_html_button::create_button(we_html_button::CANCEL, 'javascript:self.close();')
				)
			) . '</form>';
	}

	public static function getFooter($mode, we_contents_root $we_doc, $showPubl){
		switch($mode){
			default:
			case we_base_constants::MODE_NORMAL:
				return self::getFooterForNormalMode($we_doc, $showPubl);
			case we_base_constants::MODE_SEE:
				return self::getFooterForSEEMMode($we_doc, $showPubl);
		}
	}

	private static function getFooterForNormalMode(we_contents_root $we_doc, $showPubl){
		$col = 0;

		$table = new we_html_table(['class' => 'default'], 1, 0);

		$publishbutton = '';
		//	decline
		$table->addCol(2);
		$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::DECLINE, "javascript:decline_workflow();"));

		if(we_workflow_utility::isWorkflowFinished($we_doc->ID, $we_doc->Table) || ((1 + we_workflow_utility::findLastActiveStep($we_doc->ID, $we_doc->Table)) == count(we_workflow_utility::getNumberOfSteps($we_doc->ID, $we_doc->Table)) && we_base_permission::hasPerm(we_base_perm::WORK_CAN_PUBLISH))){
			$publishbutton = we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:workflow_finish();");
		} else {
			$table->addCol(2);
			$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::FORWARD, "javascript:pass_workflow();"));
		}

		if(we_workflow_utility::canUserEditDoc($we_doc->ID, $we_doc->Table, $_SESSION['user']['ID']) && $we_doc->userCanSave()){
			$table->addCol(2);
			$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::SAVE, "javascript:_EditorFrame.setEditorPublishWhenSave(false);we_save_document();"));
		}

		if($publishbutton){
			$table->addCol(2);
			$table->setColContent(0, $col++, $publishbutton);
		} elseif(we_workflow_utility::canUserEditDoc($we_doc->ID, $we_doc->Table, $_SESSION['user']['ID']) && $we_doc->userCanSave()){

			if($showPubl && (!isset($we_doc->IsClassFolder) || !$we_doc->IsClassFolder)){
				$table->addCol(2);
				$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:_EditorFrame.setEditorPublishWhenSave(true);we_save_document();"));
			}
		}

		return $table->getHtml();
	}

	private static function getFooterForSEEMMode(we_contents_root $we_doc, $showPubl){
		$col = 0;
		$table = new we_html_table(['class' => 'default'], 1, 0);

		switch($we_doc->EditPageNr){
			case we_base_constants::WE_EDITPAGE_PREVIEW:
				//	Edit-Button
				$table->addCol(2);
				$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:parent.editHeader.we_cmd('switch_edit_page'," . we_base_constants::WE_EDITPAGE_CONTENT . ",'" . $GLOBALS['we_transaction'] . "');"));

				//	Decline Workflow
				$table->addCol(2);
				$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::DECLINE, "javascript:decline_workflow();"));

				$table->addCol(2);
				if(we_workflow_utility::isWorkflowFinished($we_doc->ID, $we_doc->Table) || ((1 + we_workflow_utility::findLastActiveStep($we_doc->ID, $we_doc->Table)) == count(we_workflow_utility::getNumberOfSteps($we_doc->ID, $we_doc->Table)) && we_base_permission::hasPerm(we_base_perm::WORK_CAN_PUBLISH))){
					$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:workflow_finish();"));
				} else {
					$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::FORWARD, "javascript:pass_workflow();"));
				}
				break;
			case we_base_constants::WE_EDITPAGE_CONTENT:

				//	Preview Button
				$table->addCol(2);
				$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::PREVIEW, "javascript:parent.editHeader.we_cmd('switch_edit_page'," . we_base_constants::WE_EDITPAGE_PREVIEW . ",'" . $GLOBALS['we_transaction'] . "');"));

				//	Propertie-button
				$table->addCol(2);
				$table->setColContent(0, $col++, we_html_button::create_button('properties', "javascript:parent.editHeader.we_cmd('switch_edit_page'," . we_base_constants::WE_EDITPAGE_PROPERTIES . ",'" . $GLOBALS['we_transaction'] . "');"));

				//	Decline Workflow
				$table->addCol(2);
				$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::DECLINE, "javascript:decline_workflow();"));

				$table->addCol(2);
				$table->setColContent(0, $col++, (we_workflow_utility::isWorkflowFinished($we_doc->ID, $we_doc->Table) || ((1 + we_workflow_utility::findLastActiveStep($we_doc->ID, $we_doc->Table)) == count(we_workflow_utility::getNumberOfSteps($we_doc->ID, $we_doc->Table)) && we_base_permission::hasPerm(we_base_perm::WORK_CAN_PUBLISH)) ?
						we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:workflow_finish();") :
						we_html_button::create_button(we_html_button::FORWARD, "javascript:pass_workflow();"))
				);

				if(we_workflow_utility::canUserEditDoc($we_doc->ID, $we_doc->Table, $_SESSION['user']['ID']) && $we_doc->userCanSave()){
					$table->addCol(2);
					$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::SAVE, "javascript:_EditorFrame.setEditorPublishWhenSave(false);we_save_document();"));
					if($showPubl && (!isset($we_doc->IsClassFolder) || !$we_doc->IsClassFolder)){
						$table->addCol(2);
						$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:_EditorFrame.setEditorPublishWhenSave(true);we_save_document();"));
					}
				}
				break;

			case we_base_constants::WE_EDITPAGE_PROPERTIES:
				$table->addCol(2);
				$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::PREVIEW, "javascript:parent.editHeader.we_cmd('switch_edit_page'," . we_base_constants::WE_EDITPAGE_PREVIEW . ",'" . $GLOBALS['we_transaction'] . "');"));

				$table->addCol(2);
				$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::PREVIEW, "javascript:parent.editHeader.we_cmd('switch_edit_page'," . we_base_constants::WE_EDITPAGE_PREVIEW . ",'" . $GLOBALS['we_transaction'] . "');"));

				$table->addCol(2);
				$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::DECLINE, "javascript:decline_workflow();"));

				$table->addCol(2);
				$table->setColContent(0, $col++, (we_workflow_utility::isWorkflowFinished($we_doc->ID, $we_doc->Table) || ((1 + we_workflow_utility::findLastActiveStep($we_doc->ID, $we_doc->Table)) == count(we_workflow_utility::getNumberOfSteps($we_doc->ID, $we_doc->Table)) && we_base_permission::hasPerm(we_base_perm::WORK_CAN_PUBLISH)) ?
						we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:workflow_finish();") :
						we_html_button::create_button(we_html_button::FORWARD, "javascript:pass_workflow();"))
				);

				if(we_workflow_utility::canUserEditDoc($we_doc->ID, $we_doc->Table, $_SESSION['user']['ID']) && $we_doc->userCanSave()){
					$table->addCol(2);
					$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::SAVE, "javascript:_EditorFrame.setEditorPublishWhenSave(false);we_save_document();"));

					if($showPubl && (!isset($we_doc->IsClassFolder) || !$we_doc->IsClassFolder)){
						$table->addCol(2);
						$table->setColContent(0, $col++, we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:_EditorFrame.setEditorPublishWhenSave(true);we_save_document();"));
					}
				}
		}
		return $table->getHtml();
	}

	public function getHomeScreen(){
		$content = we_html_button::create_button('fat:new_workflow,fa-lg fa-gears', "javascript:we_cmd('new_workflow');", '', "", !we_base_permission::hasPerm(we_base_perm::WORKFLOW_NEW));

		return parent::getActualHomeScreen(we_modules_info::WORKFLOW, $content, '<form name="we_form">' . $this->getHiddens() . '</form>');
	}

}
